@@ -1,15 +1,17 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
 import os
-from xpra.log import Logger
-log = Logger("encoder", "util")
+
 from xpra.util import pver, print_nested_dict, engs, envbool, csv
 from xpra.os_util import bytestostr, strtobytes
+from xpra.log import Logger
+
+log = Logger("encoder", "util")
 
 MIN_VERSION = 375
 
@@ -74,7 +76,7 @@ def identify_nvidia_module_version():
             try:
                 numver.append(int(x))
             except ValueError:
-                if len(numver)==0:
+                if not numver:
                     raise
         if numver:
             log.info("NVidia driver version %s", pver(numver))
@@ -114,14 +116,20 @@ def identify_cards():
                             }
                 def pciinfo(pci):
                     i = {}
-                    for x in ("domain", "bus", "device", "pciDeviceId", "pciSubSystemId"):
+                    for nvname, pubname in {
+                        "domain"            : "domain",
+                        "bus"               : "bus",
+                        "device"            : "device",
+                        "pciDeviceId"       : "pci-device-id",
+                        "pciSubSystemId"    : "pci-subsystem-id",
+                        }.items():
                         try:
-                            i[x] = int(getattr(pci, x))
-                        except:
+                            i[pubname] = int(getattr(pci, nvname))
+                        except (ValueError, AttributeError):
                             pass
                     try:
-                        i["busId"] = bytestostr(pci.busId)
-                    except:
+                        i["bus-id"] = bytestostr(pci.busId)
+                    except AttributeError:
                         pass
                     return i
                 for prefix, prop, fn_name, args, conv in (
@@ -156,7 +164,8 @@ def identify_cards():
                             d = props
                         d[prop] = v
                     except Exception as e:
-                        log("identify_cards() cannot query %s using %s on device %i with handle %s: %s", prop, fn, i, handle, e)
+                        log("identify_cards() cannot query %s using %s on device %i with handle %s: %s",
+                            prop, fn, i, handle, e)
                         continue
                 log("identify_cards() [%i]=%s", i, props)
                 devices[i] = props
@@ -175,12 +184,12 @@ def identify_cards():
     return devices
 
 
-cards = None
+_cards = None
 def get_cards(probe=True):
-    global cards
-    if cards is None and probe:
-        cards = identify_cards()
-    return cards
+    global _cards
+    if _cards is None and probe:
+        _cards = identify_cards()
+    return _cards
 
 
 def is_blacklisted():
@@ -264,7 +273,7 @@ def get_license_keys(version=0, basefilename="nvenc"):
                     fkeys = []
                     for line in f:
                         sline = bytestostr(line.strip().rstrip(b'\r\n').strip())
-                        if len(sline) == 0:
+                        if not sline:
                             log("skipping empty line")
                             continue
                         if sline[0] in ('!', '#'):
