@@ -1,26 +1,35 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2019 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2009, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import time
+import sys
 import os
+import time
 import errno
 import socket
 
-from xpra.log import Logger
-log = Logger("network", "protocol")
 from xpra.net.common import ConnectionClosedException
 from xpra.util import envint, envbool, csv
-from xpra.os_util import WIN32, PYTHON2, POSIX
+from xpra.os_util import WIN32, PYTHON2, POSIX, LINUX
 from xpra.platform.features import TCP_OPTIONS, IP_OPTIONS, SOCKET_OPTIONS
+from xpra.log import Logger
 
+log = Logger("network", "protocol")
 
+SOCKET_CORK = envbool("XPRA_SOCKET_CORK", LINUX)
+if SOCKET_CORK:
+    try:
+        assert socket.TCP_CORK>0
+    except (AttributeError, AssertionError) as cork_e:
+        log.warn("Warning: unable to use TCP_CORK on %s", sys.platform)
+        log.warn(" %s", cork_e)
+        SOCKET_CORK = False
 SOCKET_NODELAY = envbool("XPRA_SOCKET_NODELAY", None)
 VSOCK_TIMEOUT = envint("XPRA_VSOCK_TIMEOUT", 5)
 SOCKET_TIMEOUT = envint("XPRA_SOCKET_TIMEOUT", 20)
-SSL_PEEK = PYTHON2 and envbool("XPRA_SSL_PEEK", True)
+SSL_PEEK = envbool("XPRA_SSL_PEEK", True)
 #this is more proper but would break the proxy server:
 SOCKET_SHUTDOWN = envbool("XPRA_SOCKET_SHUTDOWN", False)
 
@@ -62,12 +71,12 @@ FAMILY_STR = {}
 for x in dir(socket):
     if x.startswith("AF_"):
         try:
-            PROTOCOL_STR[getattr(socket, "AF_%s" % x)] = x
+            PROTOCOL_STR[getattr(socket, x)] = x
         except:
             pass
     if x.startswith("SOCK_"):
         try:
-            FAMILY_STR[getattr(socket, "SOCK_%s" % x)] = x
+            FAMILY_STR[getattr(socket, x)] = x
         except:
             pass
 
@@ -78,6 +87,7 @@ def set_continue_wait(v):
     continue_wait = v
 
 CAN_RETRY_EXCEPTIONS = ()
+CLOSED_EXCEPTIONS = ()
 
 def can_retry(e):
     if isinstance(e, socket.timeout):
@@ -98,20 +108,23 @@ def can_retry(e):
 
         abort = ABORT.get(code, code)
         if abort is not None:
-            errno = getattr(e, "errno", None)
-            log("can_retry: %s, args=%s, errno=%s, code=%s, abort=%s", type(e), e.args, errno, code, abort)
+            err = getattr(e, "errno", None)
+            log("can_retry: %s, args=%s, errno=%s, code=%s, abort=%s", type(e), e.args, err, code, abort)
             raise ConnectionClosedException(e)
+    if isinstance(e, CLOSED_EXCEPTIONS):
+        raise ConnectionClosedException(e)
     return False
 
-def untilConcludes(is_active_cb, can_retry, f, *a, **kw):
+def untilConcludes(is_active_cb, can_retry_cb, f, *a, **kw):
     global continue_wait
     wait = 0
     while is_active_cb():
         try:
             return f(*a, **kw)
         except Exception as e:
-            retry = can_retry(e)
-            log("untilConcludes(%s, %s, %s, %s, %s) %s, retry=%s", is_active_cb, can_retry, f, a, kw, e, retry, exc_info=True)
+            retry = can_retry_cb(e)
+            log("untilConcludes(%s, %s, %s, %s, %s) %s, retry=%s",
+                is_active_cb, can_retry_cb, f, a, kw, e, retry, exc_info=True)
             e = None
             if not retry:
                 raise
@@ -136,7 +149,7 @@ class Connection(object):
         log("Connection%s", (endpoint, socktype, info))
         self.endpoint = endpoint
         try:
-            assert type(endpoint) in (tuple, list)
+            assert isinstance(endpoint, (tuple, list))
             self.target = ":".join(str(x) for x in endpoint)
         except:
             self.target = str(endpoint)
@@ -154,6 +167,9 @@ class Connection(object):
     def set_nodelay(self, nodelay):
         pass
 
+    def set_cork(self, cork):
+        pass
+
     def is_active(self):
         return self.active
 
@@ -243,7 +259,8 @@ class TwoFileConnection(Connection):
         return self._write(self._oswrite, self._write_fd, buf)
 
     def close(self):
-        log("%s.close() close callback=%s, readable=%s, writeable=%s", self, self._close_cb, self._readable, self._writeable)
+        log("%s.close() close callback=%s, readable=%s, writeable=%s",
+            self, self._close_cb, self._readable, self._writeable)
         Connection.close(self)
         cc = self._close_cb
         if cc:
@@ -252,11 +269,11 @@ class TwoFileConnection(Connection):
             cc()
         try:
             self._readable.close()
-        except Exception as e:
+        except IOError as e:
             log("%s.close() %s", self._readable, e)
         try:
             self._writeable.close()
-        except:
+        except IOError as e:
             log("%s.close() %s", self._writeable, e)
         log("%s.close() done", self)
 
@@ -265,17 +282,18 @@ class TwoFileConnection(Connection):
 
     def get_info(self):
         d = Connection.get_info(self)
-        try:
-            d["type"] = "pipe"
-            d["pipe"] = {
-                         "read"     : {"fd" : self._read_fd},
-                         "write"    : {"fd" : self._write_fd},
-                         }
-        except:
-            pass
+        d.update({
+            "type"  : "pipe",
+            "pipe"  : {
+                "read"     : {"fd" : self._read_fd},
+                "write"    : {"fd" : self._write_fd},
+                }
+            })
         return d
 
 
+TCP_SOCKTYPES = ("tcp", "ssl", "ws", "wss")
+
 class SocketConnection(Connection):
     def __init__(self, socket, local, remote, target, socktype, info={}):
         log("SocketConnection%s", (socket, local, remote, target, socktype, info))
@@ -285,20 +303,26 @@ class SocketConnection(Connection):
         self.remote = remote
         self.protocol_type = "socket"
         self.nodelay = None
-        if type(remote)==str:
+        self.cork = None
+        if isinstance(remote, str):
             self.filename = remote
-        if SOCKET_NODELAY is not None:
+        if SOCKET_NODELAY is not None and self.socktype in TCP_SOCKTYPES:
             self.do_set_nodelay(SOCKET_NODELAY)
 
     def set_nodelay(self, nodelay):
-        if SOCKET_NODELAY is None and self.socktype in ("tcp", "ssl") and self.nodelay!=nodelay:
+        if SOCKET_NODELAY is None and self.socktype in TCP_SOCKTYPES and self.nodelay!=nodelay:
             self.do_set_nodelay(nodelay)
 
     def do_set_nodelay(self, nodelay):
-        if SOCKET_NODELAY is None and self.socktype in ("tcp", "ssl") and self.nodelay!=nodelay:
-            self._socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, nodelay)
-            self.nodelay = nodelay
-            log("changed %s socket to nodelay=%s", self.socktype, nodelay)
+        self._socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, nodelay)
+        self.nodelay = nodelay
+        log("changed %s socket to nodelay=%s", self.socktype, nodelay)
+
+    def set_cork(self, cork):
+        if SOCKET_CORK and self.socktype in TCP_SOCKTYPES and self.cork!=cork:
+            self._socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_CORK, cork)
+            self.cork = cork
+            log("changed %s socket to cork=%s", self.socktype, cork)
 
     def peek(self, n):
         return self._socket.recv(n, socket.MSG_PEEK)
@@ -313,29 +337,37 @@ class SocketConnection(Connection):
         s = self._socket
         try:
             i = self.get_socket_info()
-        except:
+        except IOError:
             i = s
         log("%s.close() for socket=%s", self, i)
         Connection.close(self)
         #meaningless for udp:
         try:
             s.settimeout(0)
-        except:
+        except IOError:
             pass
         if SOCKET_SHUTDOWN:
             try:
                 s.shutdown(socket.SHUT_RDWR)
-            except:
+            except IOError:
                 log("%s.shutdown(SHUT_RDWR)", s, exc_info=True)
         try:
             s.close()
         except EOFError:
             log("%s.close()", s, exc_info=True)
+        except IOError as e:
+            if isinstance(e, CLOSED_EXCEPTIONS):
+                log("%s.close() already closed!", s)
+            else:
+                raise
         log("%s.close() done", self)
 
     def __repr__(self):
         if self.remote:
-            return "%s %s: %s <- %s" % (self.socktype, self.protocol_type, pretty_socket(self.local), pretty_socket(self.remote))
+            return "%s %s: %s <- %s" % (
+                self.socktype, self.protocol_type,
+                pretty_socket(self.local), pretty_socket(self.remote),
+                )
         return "%s %s:%s" % (self.socktype, self.protocol_type, pretty_socket(self.local))
 
     def get_info(self):
@@ -346,7 +378,7 @@ class SocketConnection(Connection):
             si = self.get_socket_info()
             if si:
                 d["socket"] = si
-        except:
+        except socket.error:
             log.error("Error accessing socket information", exc_info=True)
         return d
 
@@ -363,13 +395,13 @@ class SocketConnection(Connection):
                 "family"        : FAMILY_STR.get(s.family, int(s.family)),
                 "type"          : PROTOCOL_STR.get(s.type, int(s.type)),
                 })
-        except Exception:
+        except AttributeError:
             log("do_get_socket_info()", exc_info=True)
         if self.nodelay is not None:
             info["nodelay"] = self.nodelay
         try:
             info["timeout"] = int(1000*(s.gettimeout() or 0))
-        except:
+        except socket.error:
             pass
         try:
             if POSIX:
@@ -397,7 +429,7 @@ class SocketConnection(Connection):
                     "SOCKET" : get_socket_options(s, socket.SOL_SOCKET, SOCKET_OPTIONS),
                     }
             if self.socktype in ("tcp", "udp", "ws", "wss", "ssl"):
-                    opts["IP"] = get_socket_options(s, socket.SOL_IP, IP_OPTIONS)
+                opts["IP"] = get_socket_options(s, socket.SOL_IP, IP_OPTIONS)
             if self.socktype in ("tcp", "ws", "wss", "ssl"):
                 opts["TCP"] = get_socket_options(s, socket.IPPROTO_TCP, TCP_OPTIONS)
             #ipv6:  IPV6_ADDR_PREFERENCES, IPV6_CHECKSUM, IPV6_DONTFRAG, IPV6_DSTOPTS, IPV6_HOPOPTS,
@@ -411,79 +443,99 @@ class SocketConnection(Connection):
 
 def get_socket_options(sock, level, options):
     opts = {}
-    try:
-        errs = []
-        for k in options:
-            opt = getattr(socket, k, None)
-            if opt is None:
-                continue
-            try:
-                v = sock.getsockopt(level, opt)
-            except Exception as e:
-                errs.append(k)
-            else:
-                if v is not None:
-                    opts[k] = v
-        if errs:
-            log.warn("Warning: failed to query %s", csv(errs))
-    except:
-        log.error("Error querying socket options:")
-        log.error(" %s", e)
+    errs = []
+    for k in options:
+        opt = getattr(socket, k, None)
+        if opt is None:
+            continue
+        try:
+            v = sock.getsockopt(level, opt)
+        except socket.error:
+            log("sock.getsockopt(%i, %s)", level, k, exc_info=True)
+            errs.append(k)
+        else:
+            if v is not None:
+                opts[k] = v
+    if errs:
+        log.warn("Warning: failed to query %s", csv(errs))
     return opts
 
 
-SSLSocket = None
-if SSL_PEEK:
-    try:
-        #this wrapper class allows us to override the normal ssl.Socket
-        #class so that we can fake peek() support by actually reading from the socket
-        #and caching the result.
-        class SSLSocket(socket._socketobject):
+class SSLPeekFile(object):
+    def __init__(self, fileobj, peeked, update_peek):
+        self.fileobj = fileobj
+        self.peeked = peeked
+        self.update_peek = update_peek
 
-            def __init__(self, sock):
-                socket._socketobject.__init__(self, _sock=sock)
-                #patch recv:
-                self.saved_recv = getattr(self, "recv")
-                setattr(self, "recv", self._recv)
-                self.saved_makefile = getattr(self, "makefile")
-                setattr(self, "makefile", self._makefile)
-                self.peeked = b""
+    def __getattr__(self, attr):
+        if attr=="readline" and self.peeked:
+            return self.readline
+        return getattr(self.fileobj, attr)
 
-            def _recv(self, bufsize, flags=0):
-                #log("_recv(%s, %#x) peeked=%i bytes", bufsize, flags, len(self.peeked))
-                peek = flags & socket.MSG_PEEK
-                if self.peeked:
-                    #we have peek data aleady
-                    if bufsize<len(self.peeked):
-                        r = self.peeked[:bufsize]
+    def readline(self, limit=-1):
+        if self.peeked:
+            newline = self.peeked.find(b"\n")
+            peeked = self.peeked
+            l = len(peeked)
+            if newline==-1:
+                if limit==-1 or limit>l:
+                    #we need to read more until we hit a newline:
+                    if limit==-1:
+                        more = self.fileobj.readline(limit)
                     else:
-                        r = self.peeked
-                    if not peek:
-                        #remove what we return from peek buffer:
-                        if bufsize<len(self.peeked):
-                            self.peeked = self.peeked[bufsize:]
-                        else:
-                            self.peeked = b""
-                    return r
-                r = self.saved_recv(bufsize, flags & (0xffffffff ^ socket.MSG_PEEK))
-                if peek:
-                    self.peeked = r
-                return r
+                        more = self.fileobj.readline(limit-len(self.peeked))
+                    self.peeked = b""
+                    self.update_peek(self.peeked)
+                    return peeked+more
+                read = limit
+            else:
+                if limit<0 or limit>=newline:
+                    read = newline+1
+                else:
+                    read = limit
+            self.peeked = peeked[read:]
+            self.update_peek(self.peeked)
+            return peeked[:read]
+        return self.fileobj.readline(limit)
 
-            def _makefile(self, mode='r', bufsize=-1):
-                from socket import _fileobject
-                fo = _fileobject(self, mode, bufsize)
-                return fo
+class SSLSocketWrapper(object):
+    def __init__(self, sock):
+        self.socket = sock
+        self.peeked = b""
 
-            def __repr__(self):
-                return "SSLSocket(%s)" % self._sock
+    def __getattr__(self, attr):
+        if attr=="makefile":
+            return self.makefile
+        if attr=="recv":
+            return self.recv
+        return getattr(self.socket, attr)
 
-    except Exception as e:
-        ssllog = Logger("ssl")
-        ssllog("ssl peek", exc_info=True)
-        ssllog.warn("Warning: unable to override socket object")
-        ssllog.warn(" SSL peek support will not be available")
-        ssllog.warn(" %s", e)
+    def makefile(self, mode, bufsize=None):
+        fileobj = self.socket.makefile(mode, bufsize)
+        if self.peeked and mode and mode.startswith("r"):
+            return SSLPeekFile(fileobj, self.peeked, self._update_peek)
+        return fileobj
+
+    def _update_peek(self, peeked):
+        self.peeked = peeked
+
+    def recv(self, bufsize, flags=0):
+        if flags & socket.MSG_PEEK:
+            l = len(self.peeked)
+            if l>=bufsize:
+                log("patched_recv() peeking using existing data: %i bytes", bufsize)
+                return self.peeked[:bufsize]
+            v = self.socket.recv(bufsize-l)
+            if v:
+                log("patched_recv() peeked more: %i bytes", len(v))
+                self.peeked += v
+            return self.peeked
+        if self.peeked:
+            peeked = self.peeked[:bufsize]
+            self.peeked = self.peeked[bufsize:]
+            log("patched_recv() non peek, returned already read data")
+            return peeked
+        return self.socket.recv(bufsize, flags)
 
 
 class SSLSocketConnection(SocketConnection):
@@ -494,7 +546,7 @@ class SSLSocketConnection(SocketConnection):
             reason = getattr(e, "reason", None)
             if reason in ("WRONG_VERSION_NUMBER", "UNEXPECTED_RECORD"):
                 return False
-        message = getattr(e, "message", None)
+        message = e.args[0]
         if message in SSLSocketConnection.SSL_TIMEOUT_MESSAGES:
             return True
         code = getattr(e, "code", None)
@@ -503,8 +555,8 @@ class SSLSocketConnection(SocketConnection):
         return SocketConnection.can_retry(self, e)
 
     def enable_peek(self):
-        if SSLSocket:
-            self._socket = SSLSocket(self._socket)
+        assert not isinstance(self._socket, SSLSocketWrapper)
+        self._socket = SSLSocketWrapper(self._socket)
 
     def get_info(self):
         i = SocketConnection.get_info(self)
@@ -546,14 +598,15 @@ def log_new_connection(conn, socket_info=""):
     socktype = conn.socktype
     try:
         peername = sock.getpeername()
-    except:
+    except socket.error:
         peername = str(address)
     try:
         sockname = sock.getsockname()
     except AttributeError:
         #ie: ssh channel
         sockname = ""
-    log("log_new_connection(%s, %s) type=%s, sock=%s, sockname=%s, address=%s, peername=%s", conn, socket_info, type(conn), sock, sockname, address, peername)
+    log("log_new_connection(%s, %s) type=%s, sock=%s, sockname=%s, address=%s, peername=%s",
+        conn, socket_info, type(conn), sock, sockname, address, peername)
     if peername:
         frominfo = pretty_socket(peername)
         info_msg = "New %s connection received from %s" % (socktype, frominfo)
