@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,27 +9,33 @@ import math
 import ctypes
 import struct
 import weakref
-
 import objc                         #@UnresolvedImport
 import Quartz                       #@UnresolvedImport
 import Quartz.CoreGraphics as CG    #@UnresolvedImport
-from Quartz import CGWindowListCopyWindowInfo, kCGDisplaySetModeFlag, kCGWindowListOptionOnScreenOnly, kCGNullWindowID, kCGWindowListOptionAll #@UnresolvedImport
+from Quartz import (
+    CGWindowListCopyWindowInfo, kCGDisplaySetModeFlag, kCGWindowListOptionOnScreenOnly, #@UnresolvedImport
+    kCGNullWindowID, kCGWindowListOptionAll,    #@UnresolvedImport
+    )
 from Quartz.CoreGraphics import CGDisplayRegisterReconfigurationCallback, CGDisplayRemoveReconfigurationCallback #@UnresolvedImport
 from AppKit import NSAppleEventManager, NSScreen, NSObject, NSBeep   #@UnresolvedImport
-from AppKit import NSApp, NSApplication, NSWorkspace, NSWorkspaceActiveSpaceDidChangeNotification, NSWorkspaceWillSleepNotification, NSWorkspaceDidWakeNotification     #@UnresolvedImport
+from AppKit import (
+    NSApp, NSApplication, NSWorkspace,              #@UnresolvedImport
+    NSWorkspaceActiveSpaceDidChangeNotification,    #@UnresolvedImport
+    NSWorkspaceWillSleepNotification,               #@UnresolvedImport
+    NSWorkspaceDidWakeNotification,                 #@UnresolvedImport
+    )
 from Foundation import NSUserNotification, NSUserNotificationCenter, NSUserNotificationDefaultSoundName #@UnresolvedImport
 
 from xpra.os_util import PYTHON2
 from xpra.util import envbool, envint, roundup
 from xpra.notifications.notifier_base import NotifierBase
-
 from xpra.log import Logger
+
 log = Logger("osx", "events")
 workspacelog = Logger("osx", "events", "workspace")
 mouselog = Logger("osx", "events", "mouse")
 notifylog = Logger("osx", "notify")
 
-
 OSX_FOCUS_WORKAROUND = envint("XPRA_OSX_FOCUS_WORKAROUND", 2000)
 SLEEP_HANDLER = envbool("XPRA_OSX_SLEEP_HANDLER", True)
 OSX_WHEEL_MULTIPLIER = envint("XPRA_OSX_WHEEL_MULTIPLIER", 100)
@@ -168,7 +174,7 @@ class OSX_Subprocess_Notifier(NotifierBase):
 
     def close_notify(self, nid):
         pass
-    
+
 
 def get_native_notifier_classes():
     v = []
@@ -212,6 +218,13 @@ def get_double_click_time():
         return -1
 
 
+def get_window_min_size():
+    #roughly enough to see the window buttons:
+    return 120, 1
+
+#def get_window_max_size():
+#    return 2**15-1, 2**15-1
+
 def get_window_frame_sizes():
     #use a hard-coded window position and size:
     return get_window_frame_size(20, 20, 100, 100)
