@@ -1,18 +1,18 @@
 # This file is part of Xpra.
-# Copyright (C) 2014-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2014-2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.log import Logger
-from xpra.platform.paths import get_app_dir
-log = Logger("printing")
-
 import os
 import subprocess
-from xpra.platform.win32 import constants as win32con
-from xpra.util import csv, envint
-from xpra.os_util import bytestostr
 
+from xpra.platform.win32 import constants as win32con
+from xpra.util import csv, envint, reverse_dict
+from xpra.os_util import bytestostr
+from xpra.platform.paths import get_app_dir
+from xpra.log import Logger
+
+log = Logger("printing")
 
 #allows us to skip some printers we don't want to export
 SKIPPED_PRINTERS = os.environ.get("XPRA_SKIPPED_PRINTERS", "Microsoft XPS Document Writer,Fax").split(",")
@@ -40,7 +40,7 @@ PRINTER_ENUM_VALUES = {
     "ICON7"         : 65536*7,
     "ICON8"         : 65536*8,
     }
-PRINTER_ENUM_NAMES = dict((v,k) for k,v in PRINTER_ENUM_VALUES.items())
+PRINTER_ENUM_NAMES = reverse_dict(PRINTER_ENUM_VALUES)
 log("PRINTER_ENUM_VALUES: %s", PRINTER_ENUM_VALUES)
 
 log("PRINTER_FLAGS=%s", csv(PRINTER_FLAGS))
@@ -64,7 +64,7 @@ PROCESSES = {}
 
 printers_modified_callback = None
 def init_printing(callback=None):
-    global printers_modified_callback, GSVIEW_DIR, GSPRINT_EXE, GSWINXXC_EXE
+    global printers_modified_callback
     log("init_printing(%s) printers_modified_callback=%s", callback, printers_modified_callback)
     printers_modified_callback = callback
     try:
@@ -144,7 +144,7 @@ def get_info():
     return i
 
 def get_printers():
-    global PRINTER_ENUMS, PRINTER_ENUM_VALUES, SKIPPED_PRINTERS, PRINTER_LEVEL, GSVIEW_DIR
+    global PRINTER_ENUMS, PRINTER_ENUM_VALUES, SKIPPED_PRINTERS, PRINTER_LEVEL
     printers = {}
     for penum in PRINTER_ENUMS:
         try:
@@ -183,8 +183,7 @@ def get_printers():
 
 def print_files(printer, filenames, title, options):
     log("win32.print_files%s", (printer, filenames, title, options))
-    global JOB_ID, PROCESSES, GSVIEW_DIR, GSPRINT_EXE, GSWINXXC_EXE
-    assert GSVIEW_DIR, "cannot print files without gsprint!"
+    global JOB_ID, PROCESSES
     processes = []
     for filename in filenames:
         cwd = get_app_dir()
