@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,7 +8,12 @@ import os
 import sys
 import struct
 
+from xpra.os_util import bytestostr, hexstr
+from xpra.util import iround, envbool, envint, csv
+from xpra.gtk_common.gtk_util import get_xwindow
+from xpra.os_util import is_unity, is_gnome, is_kde, is_Fedora, is_X11, is_Wayland
 from xpra.log import Logger
+
 log = Logger("posix")
 eventlog = Logger("posix", "events")
 screenlog = Logger("posix", "screen")
@@ -18,12 +23,6 @@ menulog = Logger("posix", "menu")
 mouselog = Logger("posix", "mouse")
 xinputlog = Logger("posix", "xinput")
 
-from xpra.os_util import bytestostr, hexstr
-from xpra.util import iround, envbool, envint, csv
-from xpra.gtk_common.gtk_util import get_xwindow
-from xpra.gtk_common.gobject_compat import is_gtk3
-from xpra.os_util import is_X11, is_Wayland
-
 X11WindowBindings = None
 X11XI2Bindings = None
 if is_X11():
@@ -38,7 +37,7 @@ device_bell = None
 GTK_MENUS = envbool("XPRA_GTK_MENUS", False)
 RANDR_DPI = envbool("XPRA_RANDR_DPI", True)
 XSETTINGS_DPI = envbool("XPRA_XSETTINGS_DPI", True)
-USE_NATIVE_TRAY = envbool("XPRA_USE_NATIVE_TRAY", True)
+USE_NATIVE_TRAY = envbool("XPRA_USE_NATIVE_TRAY", is_unity() or (is_gnome() and not is_Fedora()) or is_kde())
 XINPUT_WHEEL_DIV = envint("XPRA_XINPUT_WHEEL_DIV", 15)
 DBUS_SCREENSAVER = envbool("XPRA_DBUS_SCREENSAVER", False)
 
@@ -50,15 +49,7 @@ def gl_check():
 
 
 def get_native_system_tray_classes():
-    c = []
-    if USE_NATIVE_TRAY:
-        try:
-            from xpra.platform.xposix.appindicator_tray import AppindicatorTray, can_use_appindicator
-            if can_use_appindicator():
-                c.append(AppindicatorTray)
-        except Exception as e:
-            traylog("cannot load appindicator tray: %s", e)
-    return c
+    return []
 
 def get_wm_name():
     wm_name = os.environ.get("XDG_CURRENT_DESKTOP", "")
@@ -70,22 +61,31 @@ def get_wm_name():
                 traylog("_NET_SUPPORTING_WM_CHECK window=%#x", xid)
                 wm_name = _get_X11_window_property(xid, "_NET_WM_NAME", "UTF8_STRING")
                 traylog("_NET_WM_NAME=%s", wm_name)
-                return wm_name.decode("utf-8")
+                if wm_name:
+                    return wm_name.decode("utf-8")
         except Exception as e:
+            traylog("get_wm_name()", exc_info=True)
             traylog.error("Error accessing window manager information:")
             traylog.error(" %s", e)
     return wm_name
 
 def get_native_tray_classes():
     #could restrict to only DEs that have a broken system tray like "GNOME Shell"?
+    c = []
+    if USE_NATIVE_TRAY:
+        try:
+            from xpra.platform.xposix.appindicator_tray import AppindicatorTray
+            c.append(AppindicatorTray)
+        except (ImportError, ValueError):
+            traylog("cannot load appindicator tray", exc_info=True)
     if has_gtk_menu_support():  #and wm_name=="GNOME Shell":
         try:
             from xpra.platform.xposix.gtkmenu_tray import GTKMenuTray
             traylog("using GTKMenuTray for '%s' window manager", get_wm_name() or "unknown")
-            return [GTKMenuTray]
+            c.append(GTKMenuTray)
         except Exception as e:
             traylog("cannot load gtk menu tray: %s", e)
-    return get_native_system_tray_classes()
+    return c
 
 
 def get_native_notifier_classes():
@@ -172,9 +172,9 @@ def has_gtk_menu_support():
         return _has_gtk_menu_support
     try:
         from xpra.gtk_common.gtk_util import get_default_root_window
-        from xpra.x11.dbus.menu import has_gtk_menu_support
+        from xpra.x11.dbus.menu import has_gtk_menu_support as __has_gtk_menu_support
         root = get_default_root_window()
-        _has_gtk_menu_support = has_gtk_menu_support(root)
+        _has_gtk_menu_support = __has_gtk_menu_support(root)
         menulog("has_gtk_menu_support(%s)=%s", root, _has_gtk_menu_support)
     except Exception as e:
         menulog("cannot enable gtk-x11 menu support: %s", e)
@@ -271,7 +271,8 @@ def get_icc_info():
             if data:
                 screenlog("_ICC_PROFILE=%s (%s)", type(data), len(data))
                 version = _get_X11_root_property("_ICC_PROFILE_IN_X_VERSION", "CARDINAL")
-                screenlog("get_icc_info() found _ICC_PROFILE_IN_X_VERSION=%s, _ICC_PROFILE=%s", hexstr(version or ""), hexstr(data))
+                screenlog("get_icc_info() found _ICC_PROFILE_IN_X_VERSION=%s, _ICC_PROFILE=%s",
+                          hexstr(version or ""), hexstr(data))
                 icc = {
                         "source"    : "_ICC_PROFILE",
                         "data"      : data,
@@ -279,15 +280,17 @@ def get_icc_info():
                 if version:
                     try:
                         version = ord(version)
-                    except:
+                    except TypeError:
                         pass
                     icc["version"] = version
+                screenlog("get_icc_info()=%s", icc)
                 return icc
         except Exception as e:
             screenlog.error("Error: cannot access _ICC_PROFILE X11 window property")
             screenlog.error(" %s", e)
             screenlog("get_icc_info()", exc_info=True)
-    return {}
+    from xpra.platform.gui import default_get_icc_info
+    return default_get_icc_info()
 
 
 def get_antialias_info():
@@ -309,9 +312,9 @@ def get_antialias_info():
                     "hintmedium"    : 1600,
                     "hintfull"      : 2200}.get(value)
         for prop_name, name, convert in (
-                                         ("Xft/HintStyle",  "hintstyle",    str),
+                                         ("Xft/HintStyle",  "hintstyle",    bytestostr),
                                          ("Xft/HintStyle",  "contrast",     get_contrast),
-                                         ("Xft/RGBA",       "orientation",  lambda x : str(x).upper())
+                                         ("Xft/RGBA",       "orientation",  lambda x : bytestostr(x).upper())
                                          ):
             if prop_name in d:
                 value_type, value = d.get(prop_name)
@@ -384,8 +387,9 @@ def get_desktop_names():
             d = _get_X11_root_property("_NET_DESKTOP_NAMES", "UTF8_STRING")
             if d:
                 v = d.split(b"\0")
-                if len(v)>1 and v[-1]=="":
+                if len(v)>1 and v[-1]==b"":
                     v = v[:-1]
+                return [x.decode("utf8") for x in v]
         except Exception as e:
             screenlog.warn("failed to get desktop names: %s", e)
         screenlog("get_desktop_names() %s=%s", hexstr(d or ""), v)
@@ -413,11 +417,10 @@ def _get_xresources():
             from xpra.x11.gtk_x11.prop import prop_get
             from xpra.gtk_common.gtk_util import get_default_root_window
             root = get_default_root_window()
-            v = prop_get(root, "RESOURCE_MANAGER", "latin1", ignore_errors=True)
+            value = prop_get(root, "RESOURCE_MANAGER", "latin1", ignore_errors=True)
             log("RESOURCE_MANAGER=%s", v)
-            if v is None:
+            if value is None:
                 return None
-            value = v.decode("utf-8")
             #parse the resources into a dict:
             values={}
             options = value.split("\n")
@@ -438,7 +441,7 @@ def get_cursor_size():
     d = _get_xresources() or {}
     try:
         return int(d.get("Xcursor.size", 0))
-    except:
+    except ValueError:
         return -1
 
 
@@ -494,8 +497,8 @@ def system_bell(window, device, percent, _pitch, _duration, bell_class, bell_id,
 
 def _send_client_message(window, message_type, *values):
     try:
-        from xpra.x11.gtk_x11.gdk_display_source import init_display_source
-        init_display_source()
+        from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
+        init_gdk_display_source()
         from xpra.x11.bindings.window_bindings import constants #@UnresolvedImport
         X11Window = X11WindowBindings()
         root_xid = X11Window.getDefaultRootWindow()
@@ -514,7 +517,7 @@ def show_desktop(b):
     _send_client_message(None, "_NET_SHOWING_DESKTOP", int(bool(b)))
 
 def set_fullscreen_monitors(window, fsm, source_indication=0):
-    if type(fsm) not in (tuple, list):
+    if not isinstance(fsm, (tuple, list)):
         log.warn("invalid type for fullscreen-monitors: %s", type(fsm))
         return
     if len(fsm)!=4:
@@ -657,11 +660,12 @@ class XI2_Window(object):
             return
         pointer, relative_pointer, modifiers, buttons = window._pointer_modifiers(event)
         wid = self.window.get_mouse_event_wid(*pointer)
-        #log("server_input_devices=%s, server_precise_wheel=%s", client.server_input_devices, client.server_precise_wheel)
+        #log("server_input_devices=%s, server_precise_wheel=%s",
+        #    client.server_input_devices, client.server_precise_wheel)
         valuators = event.valuators
         unused_valuators = valuators.copy()
         dx, dy = 0, 0
-        if (valuators and device and device.get("enabled") and 
+        if (valuators and device and device.get("enabled") and
             client.server_input_devices=="uinput" and client.server_precise_wheel):
             XIModeRelative = 0
             classes = device.get("classes")
@@ -700,7 +704,8 @@ class XI2_Window(object):
             mv.update(event.valuators)
         #send plain motion first, if any:
         if unused_valuators:
-            xinputlog("do_xi_motion(%s, %s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s", event, device, wid, window._client._focused, window._id, event.device, pointer, modifiers, buttons)
+            xinputlog("do_xi_motion(%s, %s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s",
+                      event, device, wid, window._client._focused, window._id, event.device, pointer, modifiers, buttons)
             pdata = pointer
             if client.server_pointer_relative:
                 pdata = list(pointer)+list(relative_pointer)
@@ -760,10 +765,7 @@ class ClientExtras(object):
         if self.x11_filter:
             return
         try:
-            if is_gtk3():
-                from xpra.x11.gtk3.gdk_bindings import init_x11_filter  #@UnresolvedImport, @UnusedImport
-            else:
-                from xpra.x11.gtk2.gdk_bindings import init_x11_filter  #@UnresolvedImport, @Reimport
+            from xpra.x11.gtk_x11.gdk_bindings import init_x11_filter  #@UnresolvedImport, @UnusedImport
             self.x11_filter = init_x11_filter()
             log("x11_filter=%s", self.x11_filter)
         except Exception:
@@ -774,11 +776,8 @@ class ClientExtras(object):
     def cleanup(self):
         log("cleanup() xsettings_watcher=%s, root_props_watcher=%s", self._xsettings_watcher, self._root_props_watcher)
         if self.x11_filter:
-            if is_gtk3():
-                from xpra.x11.gtk3.gdk_bindings import cleanup_x11_filter   #@UnresolvedImport, @UnusedImport
-            else:
-                from xpra.x11.gtk2.gdk_bindings import cleanup_x11_filter   #@UnresolvedImport, @Reimport
             self.x11_filter = None
+            from xpra.x11.gtk_x11.gdk_bindings import cleanup_x11_filter   #@UnresolvedImport, @UnusedImport
             cleanup_x11_filter()
         if self._xsettings_watcher:
             self._xsettings_watcher.cleanup()
@@ -788,7 +787,8 @@ class ClientExtras(object):
             self._root_props_watcher = None
         if self.system_bus:
             bus = self.system_bus
-            log("cleanup() system bus=%s, matches: %s", bus, (self.upower_resuming_match, self.upower_sleeping_match, self.login1_match))
+            log("cleanup() system bus=%s, matches: %s",
+                bus, (self.upower_resuming_match, self.upower_sleeping_match, self.login1_match))
             self.system_bus = None
             if self.upower_resuming_match:
                 bus._clean_up_signal_match(self.upower_resuming_match)
@@ -905,9 +905,6 @@ class ClientExtras(object):
         ROOT_PROPS = ["RESOURCE_MANAGER", "_NET_WORKAREA", "_NET_CURRENT_DESKTOP"]
         try:
             self.init_x11_filter()
-            if is_gtk3():
-                #xsettings still need porting
-                return
             from xpra.gtk_common.gtk_util import get_default_root_window
             from xpra.x11.xsettings import XSettingsWatcher
             from xpra.x11.xroot_props import XRootPropWatcher
@@ -923,7 +920,9 @@ class ClientExtras(object):
                 self._root_props_watcher.do_notify("RESOURCE_MANAGER")
         except ImportError as e:
             log("do_setup_xprops%s", args, exc_info=True)
-            log.error("failed to load X11 properties/settings bindings: %s - root window properties will not be propagated", e)
+            log.error("Error: failed to load X11 properties/settings bindings:")
+            log.error(" %s", e)
+            log.error(" root window properties will not be propagated")
 
 
     def do_xi_devices_changed(self, event):
@@ -1021,12 +1020,12 @@ class ClientExtras(object):
 
 def main():
     try:
-        from xpra.x11.gtk_x11.gdk_display_source import init_display_source
-        init_display_source()
-    except:
+        from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
+        init_gdk_display_source()
+    except ImportError:
         pass
-    from xpra.platform.gui import main
-    main()
+    from xpra.platform.gui import main as gui_main
+    gui_main()
 
 
 if __name__ == "__main__":
