@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -8,18 +8,22 @@
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000041.html
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000042.html
 # (also do not import anything that imports gtk)
-import subprocess
 import sys
 import os.path
 import atexit
 import signal
-import socket
 import traceback
+from subprocess import Popen, PIPE
 
 from xpra.scripts.main import info, warn, error, no_gtk, validate_encryption, parse_env, configure_env
 from xpra.scripts.config import InitException, TRUE_OPTIONS, FALSE_OPTIONS
-from xpra.os_util import SIGNAMES, POSIX, PYTHON3, FDChangeCaptureContext, close_fds, get_ssh_port, get_username_for_uid, get_home_for_uid, get_shell_for_uid, getuid, setuidgid, get_hex_uuid, get_status_output, strtobytes, bytestostr, get_util_logger, osexpand, WIN32, OSX
-from xpra.util import envbool, csv
+from xpra.os_util import (
+    SIGNAMES, POSIX, PYTHON3, WIN32, OSX, is_Wayland,
+    FDChangeCaptureContext, close_fds, get_ssh_port,
+    get_username_for_uid, get_home_for_uid, get_shell_for_uid, getuid, setuidgid,
+    get_hex_uuid, get_status_output, strtobytes, bytestostr, get_util_logger, osexpand,
+    )
+from xpra.util import envbool, csv, nonl
 from xpra.platform.dotxpra import DotXpra
 
 
@@ -47,7 +51,7 @@ def insert_cleanup(f):
     _cleanups.insert(0, f)
 
 
-def deadly_signal(signum, _frame):
+def deadly_signal(signum):
     info("got deadly signal %s, exiting\n" % SIGNAMES.get(signum, signum))
     run_cleanups()
     # This works fine in tests, but for some reason if I use it here, then I
@@ -66,7 +70,10 @@ def _root_prop_set(prop_name, ptype="u32", value=0):
 def _root_prop_get(prop_name, ptype="u32"):
     from xpra.gtk_common.gtk_util import get_default_root_window
     from xpra.x11.gtk_x11.prop import prop_get
-    return prop_get(get_default_root_window(), prop_name, ptype)
+    try:
+        return prop_get(get_default_root_window(), prop_name, ptype)
+    except Exception:
+        return None
 
 def _save_int(prop_name, pid):
     _root_prop_set(prop_name, "u32", pid)
@@ -75,7 +82,7 @@ def _get_int(prop_name):
     return _root_prop_get(prop_name, "u32")
 
 def _save_str(prop_name, s):
-    _root_prop_set(prop_name, "latin1", s.decode("latin1"))
+    _root_prop_set(prop_name, "latin1", s)
 
 def _get_str(prop_name):
     v = _root_prop_get(prop_name, "latin1")
@@ -85,19 +92,19 @@ def _get_str(prop_name):
 
 
 def save_xvfb_pid(pid):
-    _save_int("_XPRA_SERVER_PID", pid)
+    _save_int(b"_XPRA_SERVER_PID", pid)
 
 def get_xvfb_pid():
-    return _get_int("_XPRA_SERVER_PID")
+    return _get_int(b"_XPRA_SERVER_PID")
 
 def save_dbus_pid(pid):
-    _save_int("_XPRA_DBUS_PID", pid)
+    _save_int(b"_XPRA_DBUS_PID", pid)
 
 def get_dbus_pid():
-    return _get_int("_XPRA_DBUS_PID")
+    return _get_int(b"_XPRA_DBUS_PID")
 
 def save_uinput_id(uuid):
-    _save_str("_XPRA_UINPUT_ID", uuid)
+    _save_str(b"_XPRA_UINPUT_ID", (uuid or b"").decode())
 
 #def get_uinput_id():
 #    return _get_str("_XPRA_UINPUT_ID")
@@ -112,7 +119,7 @@ def get_dbus_env():
         try:
             v = load(k)
             if v:
-                env[k] = str(v)
+                env[k] = bytestostr(v)
         except Exception as e:
             error("failed to load dbus environment variable '%s':\n" % k)
             error(" %s\n" % e)
@@ -122,8 +129,13 @@ def save_dbus_env(env):
     #DBUS_SESSION_BUS_ADDRESS=unix:abstract=/tmp/dbus-B8CDeWmam9,guid=b77f682bd8b57a5cc02f870556cbe9e9
     #DBUS_SESSION_BUS_PID=11406
     #DBUS_SESSION_BUS_WINDOWID=50331649
+    def u(s):
+        try:
+            return s.decode("latin1")
+        except:
+            return str(s)
     for n,conv,save in (
-            ("ADDRESS",     str,    _save_str),
+            ("ADDRESS",     u,    _save_str),
             ("PID",         int,    _save_int),
             ("WINDOW_ID",   int,    _save_int)):
         k = "DBUS_SESSION_BUS_%s" % n
@@ -134,6 +146,7 @@ def save_dbus_env(env):
             tv = conv(v)
             save(k, tv)
         except Exception as e:
+            get_util_logger().debug("save_dbus_env(%s)", env, exc_info=True)
             error("failed to save dbus environment variable '%s' with value '%s':\n" % (k, v))
             error(" %s\n" % e)
 
@@ -160,12 +173,14 @@ def display_name_check(display_name):
     n = display_name[1:].split(".")[0]    #ie: ":0.0" -> "0"
     try:
         dno = int(n)
-        if dno>=0 and dno<10:
+        if 0<=dno<10:
             warn("WARNING: low display number: %s" % dno)
-            warn(" You are attempting to run the xpra server against a low X11 display number: '%s'." % display_name)
+            warn(" You are attempting to run the xpra server")
+            warn(" against a low X11 display number: '%s'." % (display_name,))
             warn(" This is generally not what you want.")
-            warn(" You should probably use a higher display number just to avoid any confusion (and also this warning message).")
-    except:
+            warn(" You should probably use a higher display number")
+            warn(" just to avoid any confusion and this warning message.")
+    except IOError:
         pass
 
 def close_gtk_display():
@@ -205,8 +220,10 @@ def print_DE_warnings():
 def sanitize_env():
     def unsetenv(*varnames):
         for x in varnames:
-            if x in os.environ:
+            try:
                 del os.environ[x]
+            except KeyError:
+                pass
     #we don't want client apps to think these mean anything:
     #(if set, they belong to the desktop the server was started from)
     #TODO: simply whitelisting the env would be safer/better
@@ -215,7 +232,9 @@ def sanitize_env():
              "GNOME_DESKTOP_SESSION_ID",
              "SESSION_MANAGER",
              "XDG_VTNR",
-             "XDG_MENU_PREFIX",
+             #we must keep this value on Debian / Ubuntu
+             #to avoid breaking menu loading:
+             #"XDG_MENU_PREFIX",
              "XDG_CURRENT_DESKTOP",
              "XDG_SESSION_DESKTOP",
              "XDG_SESSION_TYPE",
@@ -299,7 +318,7 @@ def guess_xpra_display(socket_dir, socket_dirs):
     dotxpra = DotXpra(socket_dir, socket_dirs)
     results = dotxpra.sockets()
     live = [display for state, display in results if state==DotXpra.LIVE]
-    if len(live)==0:
+    if not live:
         raise InitException("no existing xpra servers found")
     if len(live)>1:
         raise InitException("too many existing xpra servers found, cannot guess which one to use")
@@ -309,29 +328,54 @@ def guess_xpra_display(socket_dir, socket_dirs):
 def start_dbus(dbus_launch):
     if not dbus_launch or dbus_launch.lower() in FALSE_OPTIONS:
         return 0, {}
+    from xpra.log import Logger
+    dbuslog = Logger("dbus")
     try:
         def preexec():
             assert POSIX
             os.setsid()
             close_fds()
-        proc = subprocess.Popen(dbus_launch, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True, preexec_fn=preexec)
-        out,_ = proc.communicate()
+        env = dict((k,v) for k,v in os.environ.items() if k in (
+            "PATH",
+            "SSH_CLIENT", "SSH_CONNECTION",
+            "XDG_CURRENT_DESKTOP", "XDG_SESSION_TYPE", "XDG_RUNTIME_DIR",
+            "SHELL", "LANG", "USER", "LOGNAME", "HOME",
+            "DISPLAY", "XAUTHORITY", "CKCON_X11_DISPLAY",
+            ))
+        import shlex
+        cmd = shlex.split(dbus_launch)
+        dbuslog("start_dbus(%s) env=%s", dbus_launch, env)
+        proc = Popen(cmd, stdin=PIPE, stdout=PIPE, shell=False, env=env, preexec_fn=preexec)
+        out = proc.communicate()[0]
         assert proc.poll()==0, "exit code is %s" % proc.poll()
         #parse and add to global env:
         dbus_env = {}
-        for l in out.splitlines():
-            parts = l.split("=", 1)
+        dbuslog("out(%s)=%s", cmd, nonl(out))
+        for l in bytestostr(out).splitlines():
+            if l.startswith("export "):
+                continue
+            sep = "="
+            if l.startswith("setenv "):
+                l = l[len("setenv "):]
+                sep = " "
+            if l.startswith("set "):
+                l = l[len("set "):]
+            parts = l.split(sep, 1)
             if len(parts)!=2:
                 continue
             k,v = parts
             if v.startswith("'") and v.endswith("';"):
                 v = v[1:-2]
+            elif v.endswith(";"):
+                v = v[:-1]
             dbus_env[k] = v
         dbus_pid = int(dbus_env.get("DBUS_SESSION_BUS_PID", 0))
+        dbuslog("dbus-env=%s", dbus_env)
         return dbus_pid, dbus_env
     except Exception as e:
-        error("dbus-launch failed to start using command '%s':\n" % dbus_launch)
-        error(" %s\n" % e)
+        dbuslog("start_dbus(%s)", dbus_launch, exc_info=True)
+        dbuslog.error("dbus-launch failed to start using command '%s':\n" % dbus_launch)
+        dbuslog.error(" %s\n" % e)
         return 0, {}
 
 
@@ -364,6 +408,41 @@ def show_encoding_help(opts):
     return 0
 
 
+def set_server_features(opts):
+    def b(v):
+        return str(v).lower() not in FALSE_OPTIONS
+    #turn off some server mixins:
+    from xpra.server import server_features
+    impwarned = []
+    def impcheck(*modules):
+        for mod in modules:
+            try:
+                __import__("xpra.%s" % mod, {}, {}, [])
+            except ImportError:
+                if mod not in impwarned:
+                    impwarned.append(mod)
+                    log = get_util_logger()
+                    log.warn("Warning: missing %s module", mod)
+                return False
+        return True
+    server_features.notifications   = opts.notifications and impcheck("notifications")
+    server_features.webcam          = b(opts.webcam) and impcheck("codecs")
+    server_features.clipboard       = b(opts.clipboard) and impcheck("clipboard")
+    server_features.audio           = (b(opts.speaker) or b(opts.microphone)) and impcheck("sound")
+    server_features.av_sync         = server_features.audio and b(opts.av_sync)
+    server_features.fileprint       = b(opts.printing) or b(opts.file_transfer)
+    server_features.mmap            = b(opts.mmap)
+    server_features.input_devices   = not opts.readonly and impcheck("keyboard")
+    server_features.commands        = impcheck("server.control_command")
+    server_features.dbus            = opts.dbus_proxy and impcheck("dbus")
+    server_features.encoding        = impcheck("codecs")
+    server_features.logging         = b(opts.remote_logging)
+    #server_features.network_state   = ??
+    server_features.display         = opts.windows
+    server_features.windows         = opts.windows and impcheck("codecs")
+    server_features.rfb             = b(opts.rfb_upgrade) and impcheck("server.rfb")
+
+
 def make_desktop_server():
     from xpra.x11.desktop_server import XpraDesktopServer
     return XpraDesktopServer()
@@ -399,7 +478,9 @@ def add_mdns(mdns_recs, socktype, host_str, port):
             recs.append(rec)
 
 def create_sockets(opts, error_cb):
-    from xpra.server.socket_util import parse_bind_ip, parse_bind_vsock, get_network_logger, setup_tcp_socket, setup_udp_socket, setup_vsock_socket
+    from xpra.server.socket_util import (
+        parse_bind_ip, parse_bind_vsock, get_network_logger, setup_tcp_socket, setup_udp_socket, setup_vsock_socket,
+        )
     log = get_network_logger()
 
     bind_tcp = parse_bind_ip(opts.bind_tcp)
@@ -476,7 +557,7 @@ def create_sockets(opts, error_cb):
     for host, iport in bind_ssl:
         add_tcp_socket("ssl", host, iport)
         if ws_upgrades:
-            add_tcp_socket("wss", host, iport)
+            add_mdns(mdns_recs, "wss", host, iport)
     log("setting up SSH sockets: %s", csv(bind_ssh))
     for host, iport in bind_ssh:
         add_tcp_socket("ssh", host, iport)
@@ -532,25 +613,43 @@ def create_sockets(opts, error_cb):
                     add_mdns(mdns_recs, "tcp", host, iport)
     return sockets, mdns_recs, wrap_socket_fn
 
-
 def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None):
+    #add finally hook to ensure we will run the cleanups
+    #even if we exit because of an exception:
+    try:
+        return do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display)
+    finally:
+        run_cleanups()
+        import gc
+        gc.collect()
+
+
+def do_run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None):
     try:
         cwd = os.getcwd()
-    except:
+    except OSError:
         cwd = os.path.expanduser("~")
         warn("current working directory does not exist, using '%s'\n" % cwd)
     validate_encryption(opts)
     if opts.encoding=="help" or "help" in opts.encodings:
         return show_encoding_help(opts)
 
-    assert mode in ("start", "start-desktop", "upgrade", "shadow", "proxy")
+    assert mode in (
+        "start", "start-desktop",
+        "upgrade", "upgrade-desktop",
+        "shadow", "proxy",
+        )
     starting  = mode == "start"
     starting_desktop = mode == "start-desktop"
     upgrading = mode == "upgrade"
+    upgrading_desktop = mode == "upgrade-desktop"
     shadowing = mode == "shadow"
     proxying  = mode == "proxy"
-    clobber   = upgrading or opts.use_display
-    start_vfb = not shadowing and not proxying and not clobber
+    clobber   = upgrading or upgrading_desktop or opts.use_display
+    start_vfb = not (shadowing or proxying or clobber)
+
+    if shadowing and is_Wayland():
+        warn("shadow servers do not support Wayland, switch to X11")
 
     if opts.bind_rfb and (proxying or starting):
         get_util_logger().warn("Warning: bind-rfb sockets cannot be used with '%s' mode" % mode)
@@ -559,12 +658,12 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
     if not shadowing and not starting_desktop:
         opts.rfb_upgrade = 0
 
-    if upgrading or shadowing:
+    if upgrading or upgrading_desktop or shadowing:
         #there should already be one running
         opts.pulseaudio = False
 
     #get the display name:
-    if shadowing and len(extra_args)==0:
+    if shadowing and not extra_args:
         if WIN32 or OSX:
             #just a virtual name for the only display available:
             display_name = ":0"
@@ -572,7 +671,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
             from xpra.scripts.main import guess_X11_display
             dotxpra = DotXpra(opts.socket_dir, opts.socket_dirs)
             display_name = guess_X11_display(dotxpra, desktop_display)
-    elif upgrading and len(extra_args)==0:
+    elif (upgrading or upgrading_desktop) and not extra_args:
         display_name = guess_xpra_display(opts.socket_dir, opts.socket_dirs)
     else:
         if len(extra_args) > 1:
@@ -604,14 +703,21 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
                 # Use the temporary magic value 'S' as marker:
                 display_name = 'S' + str(os.getpid())
 
-    if not shadowing and not proxying and not upgrading and opts.exit_with_children and not opts.start_child:
+    if not (shadowing or proxying or upgrading or upgrading_desktop) and \
+    opts.exit_with_children and not opts.start_child:
         error_cb("--exit-with-children specified without any children to spawn; exiting immediately")
 
     atexit.register(run_cleanups)
 
     # Generate the script text now, because os.getcwd() will
     # change if/when we daemonize:
-    from xpra.server.server_util import xpra_runner_shell_script, write_runner_shell_scripts, write_pidfile, find_log_dir, create_input_devices
+    from xpra.server.server_util import (
+        xpra_runner_shell_script,
+        write_runner_shell_scripts,
+        write_pidfile,
+        find_log_dir,
+        create_input_devices,
+        )
     script = xpra_runner_shell_script(xpra_file, cwd, opts.socket_dir)
 
     uid = int(opts.uid)
@@ -742,7 +848,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
                 stderr.write("Entering daemon mode; "
                          + "any further errors will be reported to:\n"
                          + ("  %s\n" % log_filename0))
-            except:
+            except IOError:
                 #we tried our best, logging another error won't help
                 pass
 
@@ -750,7 +856,8 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
     if (starting or starting_desktop) and desktop_display and opts.notifications and not opts.dbus_launch:
         print_DE_warnings()
 
-    log = get_util_logger()
+    from xpra.log import Logger
+    log = Logger("server")
     sockets, mdns_recs, wrap_socket_fn = create_sockets(opts, error_cb)
 
     sanitize_env()
@@ -767,7 +874,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
     else:
         try:
             del os.environ["DISPLAY"]
-        except:
+        except KeyError:
             pass
     os.environ.update(protected_env)
     log("env=%s", os.environ)
@@ -825,9 +932,14 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
             log.error(" %s", str(e) or type(e))
             del e
         try:
-            os.close(displayfd)
-        except:
-            pass
+            os.fsync(displayfd)
+        except (IOError, OSError):
+            log("os.fsync(%i)", displayfd, exc_info=True)
+        if displayfd>2:
+            try:
+                os.close(displayfd)
+            except (IOError, OSError):
+                log("os.close(%i)", displayfd, exc_info=True)
 
     kill_display = None
     if not proxying:
@@ -842,7 +954,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
         def noerr(fn, *args):
             try:
                 fn(*args)
-            except:
+            except Exception:
                 pass
         log_filename1 = osexpand(select_log_file(log_dir, opts.log_file, display_name), username, uid, gid)
         if log_filename0 != log_filename1:
@@ -911,21 +1023,51 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
     if opts.chdir:
         os.chdir(opts.chdir)
 
+    dbus_pid = 0
+    dbus_env = {}
+    kill_dbus = None
+    if not shadowing and POSIX and not clobber:
+        no_gtk()
+        dbuslog = Logger("dbus")
+        assert starting or starting_desktop or proxying
+        bus_address = protected_env.get("DBUS_SESSION_BUS_ADDRESS")
+        dbuslog("dbus_launch=%s, current DBUS_SESSION_BUS_ADDRESS=%s", opts.dbus_launch, bus_address)
+        if opts.dbus_launch and not bus_address:
+            #start a dbus server:
+            def kill_dbus():
+                dbuslog("kill_dbus: dbus_pid=%s" % dbus_pid)
+                if dbus_pid<=0:
+                    return
+                try:
+                    os.kill(dbus_pid, signal.SIGINT)
+                except Exception as e:
+                    dbuslog("os.kill(%i, SIGINT)", dbus_pid, exc_info=True)
+                    dbuslog.warn("Warning: error trying to stop dbus with pid %i:", dbus_pid)
+                    dbuslog.warn(" %s", e)
+            add_cleanup(kill_dbus)
+            #this also updates os.environ with the dbus attributes:
+            dbus_pid, dbus_env = start_dbus(opts.dbus_launch)
+            dbuslog("dbus attributes: pid=%s, env=%s", dbus_pid, dbus_env)
+            if dbus_env:
+                os.environ.update(dbus_env)
+                os.environ.update(protected_env)
+
     display = None
     if not proxying:
-        no_gtk()
-        if POSIX and not OSX and (starting or starting_desktop or shadowing):
-            #check that we can access the X11 display:
-            from xpra.x11.vfb_util import verify_display_ready
-            if not verify_display_ready(xvfb, display_name, shadowing):
-                return 1
-            if not PYTHON3:
-                from xpra.x11.gtk2.gdk_display_util import verify_gdk_display       #@UnusedImport
-            else:
-                from xpra.x11.gtk3.gdk_display_util import verify_gdk_display       #@Reimport
-            display = verify_gdk_display(display_name)
-            if not display:
-                return 1
+        if POSIX and not OSX:
+            no_gtk()
+            if starting or starting_desktop or shadowing:
+                #check that we can access the X11 display:
+                from xpra.x11.vfb_util import verify_display_ready
+                if not verify_display_ready(xvfb, display_name, shadowing):
+                    return 1
+                if not PYTHON3:
+                    from xpra.x11.gtk2.gdk_display_util import verify_gdk_display       #@UnusedImport
+                else:
+                    from xpra.x11.gtk3.gdk_display_util import verify_gdk_display       #@Reimport
+                display = verify_gdk_display(display_name)
+                if not display:
+                    return 1
         #on win32, this ensures that we get the correct screen size to shadow:
         from xpra.platform.gui import init as gui_init
         gui_init()
@@ -937,7 +1079,11 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
         #we always need at least one valid socket dir
         from xpra.platform.paths import get_socket_dirs
         opts.socket_dirs = get_socket_dirs()
-    local_sockets = setup_local_sockets(opts.bind, opts.socket_dir, opts.socket_dirs, display_name, clobber, opts.mmap_group, opts.socket_permissions, username, uid, gid)
+    local_sockets = setup_local_sockets(opts.bind,
+                                        opts.socket_dir, opts.socket_dirs,
+                                        display_name, clobber,
+                                        opts.mmap_group, opts.socket_permissions,
+                                        username, uid, gid)
     netlog("setting up local sockets: %s", local_sockets)
     ssh_port = get_ssh_port()
     ssh_access = ssh_port>0 and opts.ssh.lower().strip() not in FALSE_OPTIONS
@@ -951,40 +1097,8 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
             netlog("ssh %s:%s : %s", "", ssh_port, socket)
             add_mdns(mdns_recs, "ssh", "", ssh_port)
 
-    def b(v):
-        return str(v).lower() not in FALSE_OPTIONS
-    #turn off some server mixins:
-    from xpra.server import server_features
-    impwarned = []
-    def impcheck(*modules):
-        for mod in modules:
-            try:
-                __import__("xpra.%s" % mod, {}, {}, [])
-            except ImportError as e:
-                if mod not in impwarned:
-                    impwarned.append(mod)
-                    log = get_util_logger()
-                    log.warn("Warning: missing %s module", mod)
-                return False
-        return True
-    server_features.notifications   = opts.notifications and impcheck("notifications")
-    server_features.webcam          = b(opts.webcam) and impcheck("codecs")
-    server_features.clipboard       = b(opts.clipboard) and impcheck("clipboard")
-    server_features.audio           = (b(opts.speaker) or b(opts.microphone)) and impcheck("sound")
-    server_features.av_sync         = server_features.audio and b(opts.av_sync)
-    server_features.fileprint       = b(opts.printing) or b(opts.file_transfer)
-    server_features.mmap            = b(opts.mmap)
-    server_features.input_devices   = not opts.readonly and impcheck("keyboard")
-    server_features.commands        = impcheck("server.control_command")
-    server_features.dbus            = opts.dbus_proxy and impcheck("dbus")
-    server_features.encoding        = impcheck("codecs")
-    server_features.logging         = b(opts.remote_logging)
-    #server_features.network_state   = ??
-    server_features.display         = opts.windows
-    server_features.windows         = opts.windows and impcheck("codecs")
-    server_features.rfb             = b(opts.rfb_upgrade) and impcheck("server.rfb")
+    set_server_features(opts)
 
-    kill_dbus = None
     if shadowing:
         app = make_shadow_server()
     elif proxying:
@@ -992,8 +1106,8 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
     else:
         if not check_xvfb():
             return  1
-        assert starting or starting_desktop or upgrading
-        from xpra.x11.gtk2.gdk_display_source import init_gdk_display_source, close_gdk_display_source
+        assert starting or starting_desktop or upgrading or upgrading_desktop
+        from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source, close_gdk_display_source
         init_gdk_display_source()
         insert_cleanup(close_gdk_display_source)
         #(now we can access the X11 server)
@@ -1001,48 +1115,25 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
         #make sure the pid we save is the real one:
         if not check_xvfb():
             return  1
-        if xvfb_pid is not None:
-            #save the new pid (we should have one):
-            save_xvfb_pid(xvfb_pid)
 
-        if POSIX:
-            save_uinput_id(uinput_uuid or "")
-            dbus_pid = -1
-            dbus_env = {}
-            if clobber:
-                #get the saved pids and env
-                dbus_pid = get_dbus_pid()
-                dbus_env = get_dbus_env()
-                log("retrieved existing dbus attributes")
-            else:
-                assert starting or starting_desktop
-                if xvfb_pid is not None:
-                    #save the new pid (we should have one):
-                    save_xvfb_pid(xvfb_pid)
-                bus_address = protected_env.get("DBUS_SESSION_BUS_ADDRESS")
-                log("dbus_launch=%s, current DBUS_SESSION_BUS_ADDRESS=%s", opts.dbus_launch, bus_address)
-                if opts.dbus_launch and not bus_address:
-                    #start a dbus server:
-                    def kill_dbus():
-                        log("kill_dbus: dbus_pid=%s" % dbus_pid)
-                        if dbus_pid<=0:
-                            return
-                        try:
-                            os.kill(dbus_pid, signal.SIGINT)
-                        except Exception as e:
-                            log.warn("Warning: error trying to stop dbus with pid %i:", dbus_pid)
-                            log.warn(" %s", e)
-                    add_cleanup(kill_dbus)
-                    #this also updates os.environ with the dbus attributes:
-                    dbus_pid, dbus_env = start_dbus(opts.dbus_launch)
-                    if dbus_pid>0:
-                        save_dbus_pid(dbus_pid)
-                    if dbus_env:
-                        save_dbus_env(dbus_env)
-            log("dbus attributes: pid=%s, env=%s", dbus_pid, dbus_env)
+        #now we can save values on the display
+        #(we cannot access gtk3 until dbus has started up)
+        if not clobber:
+            if xvfb_pid:
+                save_xvfb_pid(xvfb_pid)
+            if dbus_pid:
+                save_dbus_pid(dbus_pid)
+            if dbus_env:
+                save_dbus_env(dbus_env)
+        else:
+            #get the saved pids and env
+            dbus_pid = get_dbus_pid()
+            dbus_env = get_dbus_env()
+            dbuslog = Logger("dbus")
+            dbuslog("retrieved existing dbus attributes: %s, %s", dbus_pid, dbus_env)
             if dbus_env:
                 os.environ.update(dbus_env)
-                os.environ.update(protected_env)
+        save_uinput_id(uinput_uuid)
 
         if POSIX:
             #all unix domain sockets:
@@ -1054,7 +1145,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
                         xdg_override = os.path.join(x, "xdg-open")
                         if os.path.exists(xdg_override):
                             os.environ["PATH"] = x+os.pathsep+os.environ.get("PATH", "")
-                            os.environ["XPRA_XDG_OPEN_SERVER_SOCKET"] = ud_paths[0]
+                            os.environ["XPRA_SERVER_SOCKET"] = ud_paths[0]
                             break
             else:
                 log.warn("Warning: no local server sockets,")
@@ -1078,22 +1169,21 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
                 log.error("Xpra 'start' subcommand runs as a compositing manager")
                 log.error(" it cannot use a display which lacks the XComposite extension!")
                 return 1
-            assert not PYTHON3
             if starting:
                 #check for an existing window manager:
-                from xpra.x11.gtk2.wm import wm_check
+                from xpra.x11.gtk_x11.wm_check import wm_check
                 if not wm_check(display, opts.wm_name, upgrading):
                     return 1
             log("XShape=%s", X11Window.displayHasXShape())
             app = make_server(clobber)
         else:
-            assert starting_desktop and not PYTHON3
+            assert starting_desktop or upgrading_desktop
             app = make_desktop_server()
         app.init_virtual_devices(devices)
 
-    if proxying or upgrading:
+    if proxying or upgrading or upgrading_desktop:
         #when proxying or upgrading, don't exec any plain start commands:
-        opts.start = opts.start_child = [] 
+        opts.start = opts.start_child = []
     elif opts.exit_with_children:
         assert opts.start_child, "exit-with-children was specified but start-child is missing!"
     elif opts.start_child:
@@ -1131,8 +1221,8 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
         MDNS_EXPOSE_NAME = envbool("XPRA_MDNS_EXPOSE_NAME", True)
         if MDNS_EXPOSE_NAME and app.session_name:
             mdns_info["name"] = app.session_name
-        for mode, listen_on in mdns_recs.items():
-            mdns_publish(display_name, mode, listen_on, mdns_info)
+        for mdns_mode, listen_on in mdns_recs.items():
+            mdns_publish(display_name, mdns_mode, listen_on, mdns_info)
 
     del opts
 
@@ -1143,7 +1233,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
 
     try:
         #from here on, we own the vfb, even if we inherited one:
-        if (starting or starting_desktop or upgrading) and clobber:
+        if (starting or starting_desktop or upgrading or upgrading_desktop) and clobber:
             #and it will be killed if exit cleanly:
             xvfb_pid = get_xvfb_pid()
 
@@ -1152,26 +1242,21 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
         log("%s()=%s", app.run, r)
     except KeyboardInterrupt:
         log.info("stopping on KeyboardInterrupt")
-        r = 0
+        return 0
     except Exception:
         log.error("server error", exc_info=True)
-        r = -128
-    if r>0:
-        # Upgrading/exiting, so leave X and dbus servers running
-        if kill_display:
-            _cleanups.remove(kill_display)
-        if kill_dbus:
-            _cleanups.remove(kill_dbus)
-        from xpra.server import EXITING_CODE
-        if r==EXITING_CODE:
-            log.info("exiting: not cleaning up Xvfb")
-        else:
-            log.info("upgrading: not cleaning up Xvfb")
-        r = 0
-    log("cleanups=%s", _cleanups)
-    try:
-        return r
-    finally:
-        run_cleanups()
-        import gc
-        gc.collect()
+        return -128
+    else:
+        if r>0:
+            # Upgrading/exiting, so leave X and dbus servers running
+            if kill_display:
+                _cleanups.remove(kill_display)
+            if kill_dbus:
+                _cleanups.remove(kill_dbus)
+            from xpra.server import EXITING_CODE
+            if r==EXITING_CODE:
+                log.info("exiting: not cleaning up Xvfb")
+            else:
+                log.info("upgrading: not cleaning up Xvfb")
+            r = 0
+    return r
