@@ -1,24 +1,22 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os.path
 
-from xpra.log import Logger
-log = Logger("server")
-log = Logger("exec")
-
 from xpra.platform.features import COMMAND_SIGNALS
 from xpra.child_reaper import getChildReaper, reaper_cleanup
-from xpra.os_util import monotonic_time, OSX, WIN32
+from xpra.os_util import monotonic_time, OSX, WIN32, POSIX
 from xpra.util import envint, csv
 from xpra.scripts.parsing import parse_env
 from xpra.server import EXITING_CODE
 from xpra.server.mixins.stub_server_mixin import StubServerMixin
+from xpra.log import Logger
 
+log = Logger("exec")
 
 TERMINATE_DELAY = envint("XPRA_TERMINATE_DELAY", 1000)/1000.0
 
@@ -37,6 +35,8 @@ class ChildCommandServer(StubServerMixin):
         self.start_child_after_connect = []
         self.start_on_connect = []
         self.start_child_on_connect = []
+        self.start_on_last_client_exit = []
+        self.start_child_on_last_client_exit = []
         self.exit_with_children = False
         self.start_after_connect_done = False
         self.start_new_commands = False
@@ -57,6 +57,8 @@ class ChildCommandServer(StubServerMixin):
         self.start_child_after_connect   = opts.start_child_after_connect
         self.start_on_connect            = opts.start_on_connect
         self.start_child_on_connect      = opts.start_child_on_connect
+        self.start_on_last_client_exit   = opts.start_on_last_client_exit
+        self.start_child_on_last_client_exit = opts.start_child_on_last_client_exit
         if opts.exec_wrapper:
             import shlex
             self.exec_wrapper = shlex.split(opts.exec_wrapper)
@@ -87,6 +89,22 @@ class ChildCommandServer(StubServerMixin):
             "server-commands-info"      : not WIN32 and not OSX,
             }
 
+    def get_caps(self, source):
+        caps = {}
+        if self.start_new_commands and POSIX and not OSX and source.wants_features:
+            from xpra.platform.xposix.xdg_helper import load_xdg_menu_data, remove_icons
+            xdg_menu = load_xdg_menu_data()
+            if xdg_menu:
+                l = len(str(xdg_menu))
+                #arbitrary: don't use more than half
+                #of the maximum size of the hello packet:
+                if l>2*1024*1024:
+                    xdg_menu = remove_icons(xdg_menu)
+                    log.info("removed icons to reduce the size of the xdg menu data")
+                    log.info("size reduced from %i to %i", l, len(str(xdg_menu)))
+                caps["xdg-menu"] = xdg_menu
+        return caps
+
 
     def get_info(self, _proto):
         info = {
@@ -105,6 +123,10 @@ class ChildCommandServer(StubServerMixin):
         return {"commands": info}
 
 
+    def last_client_exited(self):
+        self._exec_commands(self.start_on_last_client_exit, self.start_child_on_last_client_exit)
+
+
     def get_child_env(self):
         #subclasses may add more items (ie: fakexinerama)
         env = os.environ.copy()
@@ -116,7 +138,7 @@ class ChildCommandServer(StubServerMixin):
 
     def get_full_child_command(self, cmd, use_wrapper=True):
         #make sure we have it as a list:
-        if type(cmd) not in (list, tuple):
+        if not isinstance(cmd, (list, tuple)):
             import shlex
             cmd = shlex.split(str(cmd))
         if not use_wrapper or not self.exec_wrapper:
@@ -129,7 +151,8 @@ class ChildCommandServer(StubServerMixin):
         self._exec_commands(self.start_commands, self.start_child_commands)
 
     def exec_after_connect_commands(self):
-        log("exec_after_connect_commands() start=%s, start_child=%s", self.start_after_connect, self.start_child_after_connect)
+        log("exec_after_connect_commands() start=%s, start_child=%s",
+            self.start_after_connect, self.start_child_after_connect)
         self._exec_commands(self.start_after_connect, self.start_child_after_connect)
 
     def exec_on_connect_commands(self):
@@ -155,13 +178,14 @@ class ChildCommandServer(StubServerMixin):
             self.idle_add(self.guess_session_name, procs)
 
     def start_command(self, name, child_cmd, ignore=False, callback=None, use_wrapper=True, shell=False, **kwargs):
-        log("start_command%s exec_wrapper=%s", (name, child_cmd, ignore, callback, use_wrapper, shell, kwargs), self.exec_wrapper)
-        import subprocess
+        log("start_command%s exec_wrapper=%s",
+            (name, child_cmd, ignore, callback, use_wrapper, shell, kwargs), self.exec_wrapper)
+        from subprocess import Popen, PIPE
         env = self.get_child_env()
         try:
             real_cmd = self.get_full_child_command(child_cmd, use_wrapper)
             log("full child command(%s, %s)=%s", child_cmd, use_wrapper, real_cmd)
-            proc = subprocess.Popen(real_cmd, stdin=subprocess.PIPE, env=env, shell=shell, cwd=self.exec_cwd, close_fds=True, **kwargs)
+            proc = Popen(real_cmd, stdin=PIPE, env=env, shell=shell, cwd=self.exec_cwd, close_fds=True, **kwargs)
             procinfo = self.add_process(proc, name, real_cmd, ignore=ignore, callback=callback)
             log("pid(%s)=%s", real_cmd, proc.pid)
             if not ignore:
@@ -169,7 +193,9 @@ class ChildCommandServer(StubServerMixin):
             self.children_started.append(procinfo)
             return proc
         except OSError as e:
-            log.error("Error spawning child '%s': %s\n" % (child_cmd, e))
+            log("start_command%s", (name, child_cmd, ignore, callback, use_wrapper, shell, kwargs), exc_info=True)
+            log.error("Error spawning child '%s':" % (child_cmd, ))
+            log.error(" %s" % (e,))
             return None
 
 
@@ -242,7 +268,7 @@ class ChildCommandServer(StubServerMixin):
             log.warn(" but the feature is currently disabled")
             return
         name, command, ignore = packet[1:4]
-        proc = self.start_command(name, command, ignore)
+        proc = self.start_command(name.decode("utf-8"), command.decode("utf-8"), ignore)
         if len(packet)>=5:
             shared = packet[4]
             if proc and not shared:
@@ -254,7 +280,7 @@ class ChildCommandServer(StubServerMixin):
 
     def _process_command_signal(self, _proto, packet):
         pid = packet[1]
-        signame = packet[2]
+        signame = bytestostr(packet[2])
         if signame not in COMMAND_SIGNALS:
             log.warn("Warning: invalid signal received: '%s'", signame)
             return
