@@ -1,52 +1,83 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+from threading import Thread
 from time import sleep
 
-from xpra.log import Logger
-log = Logger("server")
-netlog = Logger("network")
-notifylog = Logger("notify")
-httplog = Logger("http")
-timeoutlog = Logger("timeout")
-
 from xpra.server.server_core import ServerCore, get_thread_info
 from xpra.server.mixins.server_base_controlcommands import ServerBaseControlCommands
-from xpra.server.mixins.notification_forwarder import NotificationForwarder
-from xpra.server.mixins.webcam_server import WebcamServer
-from xpra.server.mixins.clipboard_server import ClipboardServer
-from xpra.server.mixins.audio_server import AudioServer
-from xpra.server.mixins.fileprint_server import FilePrintServer
-from xpra.server.mixins.mmap_server import MMAP_Server
-from xpra.server.mixins.input_server import InputServer
-from xpra.server.mixins.child_command_server import ChildCommandServer
-from xpra.server.mixins.dbusrpc_server import DBUS_RPC_Server
-from xpra.server.mixins.encoding_server import EncodingServer
-from xpra.server.mixins.logging_server import LoggingServer
-from xpra.server.mixins.networkstate_server import NetworkStateServer
-from xpra.server.mixins.display_manager import DisplayManager
-from xpra.server.mixins.window_server import WindowServer
-
-from xpra.os_util import thread, monotonic_time, bytestostr, WIN32, PYTHON3
-from xpra.util import typedict, flatten_dict, updict, merge_dicts, envbool, envint, \
-    SERVER_EXIT, SERVER_ERROR, SERVER_SHUTDOWN, DETACH_REQUEST, NEW_CLIENT, DONE, SESSION_BUSY, XPRA_NEW_USER_NOTIFICATION_ID
+from xpra.os_util import monotonic_time, bytestostr, strtobytes, WIN32, PYTHON3
+from xpra.util import (
+    typedict, flatten_dict, updict, merge_dicts, envbool, envint,
+    SERVER_EXIT, SERVER_ERROR, SERVER_SHUTDOWN, DETACH_REQUEST,
+    NEW_CLIENT, DONE, SESSION_BUSY,
+    )
 from xpra.net.bytestreams import set_socket_timeout
-from xpra.platform.paths import get_icon_filename
-from xpra.notifications.common import parse_image_path
+from xpra.server import server_features
 from xpra.server import EXITING_CODE
-from xpra.codecs.loader import codec_versions
+from xpra.log import Logger
 
+SERVER_BASES = [ServerCore, ServerBaseControlCommands]
+if server_features.notifications:
+    from xpra.server.mixins.notification_forwarder import NotificationForwarder
+    SERVER_BASES.append(NotificationForwarder)
+if server_features.webcam:
+    from xpra.server.mixins.webcam_server import WebcamServer
+    SERVER_BASES.append(WebcamServer)
+if server_features.clipboard:
+    from xpra.server.mixins.clipboard_server import ClipboardServer
+    SERVER_BASES.append(ClipboardServer)
+if server_features.audio:
+    from xpra.server.mixins.audio_server import AudioServer
+    SERVER_BASES.append(AudioServer)
+if server_features.fileprint:
+    from xpra.server.mixins.fileprint_server import FilePrintServer
+    SERVER_BASES.append(FilePrintServer)
+if server_features.mmap:
+    from xpra.server.mixins.mmap_server import MMAP_Server
+    SERVER_BASES.append(MMAP_Server)
+if server_features.input_devices:
+    from xpra.server.mixins.input_server import InputServer
+    SERVER_BASES.append(InputServer)
+if server_features.commands:
+    from xpra.server.mixins.child_command_server import ChildCommandServer
+    SERVER_BASES.append(ChildCommandServer)
+if server_features.dbus:
+    from xpra.server.mixins.dbusrpc_server import DBUS_RPC_Server
+    SERVER_BASES.append(DBUS_RPC_Server)
+if server_features.encoding:
+    from xpra.server.mixins.encoding_server import EncodingServer
+    SERVER_BASES.append(EncodingServer)
+if server_features.logging:
+    from xpra.server.mixins.logging_server import LoggingServer
+    SERVER_BASES.append(LoggingServer)
+if server_features.network_state:
+    from xpra.server.mixins.networkstate_server import NetworkStateServer
+    SERVER_BASES.append(NetworkStateServer)
+if server_features.display:
+    from xpra.server.mixins.display_manager import DisplayManager
+    SERVER_BASES.append(DisplayManager)
+if server_features.windows:
+    from xpra.server.mixins.window_server import WindowServer
+    SERVER_BASES.append(WindowServer)
+SERVER_BASES = tuple(SERVER_BASES)
+ServerBaseClass = type('ServerBaseClass', SERVER_BASES, {})
+
+log = Logger("server")
+netlog = Logger("network")
+httplog = Logger("http")
+timeoutlog = Logger("timeout")
+screenlog = Logger("screen")
+
+log("ServerBaseClass%s", SERVER_BASES)
 
-DETECT_MEMLEAKS = envbool("XPRA_DETECT_MEMLEAKS", False)
-DETECT_FDLEAKS = envbool("XPRA_DETECT_FDLEAKS", False)
-MAX_CONCURRENT_CONNECTIONS = 20
 CLIENT_CAN_SHUTDOWN = envbool("XPRA_CLIENT_CAN_SHUTDOWN", True)
-TERMINATE_DELAY = envint("XPRA_TERMINATE_DELAY", 1000)/1000.0
+INIT_THREAD_TIMEOUT = envint("XPRA_INIT_THREAD_TIMEOUT", 10)
 
 
 """
@@ -55,10 +86,10 @@ It provides all the generic functions but is not tied
 to a specific backend (X11 or otherwise).
 See GTKServerBase/X11ServerBase and other platform specific subclasses.
 """
-class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, WebcamServer, ClipboardServer, AudioServer, FilePrintServer, MMAP_Server, InputServer, ChildCommandServer, DBUS_RPC_Server, EncodingServer, LoggingServer, NetworkStateServer, DisplayManager, WindowServer):
+class ServerBase(ServerBaseClass):
 
     def __init__(self):
-        for c in ServerBase.__bases__:
+        for c in SERVER_BASES:
             c.__init__(self)
         log("ServerBase.__init__()")
         self.init_uuid()
@@ -71,10 +102,10 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
         self.ui_driver = None
         self.sharing = None
         self.lock = None
+        self.init_thread = None
 
         self.idle_timeout = 0
         #duplicated from Server Source...
-        self.mem_bytes = 0
         self.client_shutdown = CLIENT_CAN_SHUTDOWN
         self.mp3_stream_check_timer = None
 
@@ -103,7 +134,7 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
         #from now on, use the logger for parsing errors:
         from xpra.scripts import config
         config.warn = log.warn
-        for c in ServerBase.__bases__:
+        for c in SERVER_BASES:
             c.init(self, opts)
         log("ServerBase.init(%s)", opts)
 
@@ -111,22 +142,34 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
         self.lock = opts.lock
         self.idle_timeout = opts.idle_timeout
         self.av_sync = opts.av_sync
+        self.bandwidth_detection = opts.bandwidth_detection
 
 
     def setup(self):
         log("starting component init")
-        for c in ServerBase.__bases__:
+        for c in SERVER_BASES:
             c.setup(self)
-        thread.start_new_thread(self.threaded_init, ())
+        self.init_thread = Thread(target=self.threaded_init)
+        self.init_thread.start()
 
     def threaded_init(self):
         log("threaded_init() start")
-        sleep(0.1)
-        for c in ServerBase.__bases__:
+        from xpra.platform import threaded_server_init
+        threaded_server_init()
+        for c in SERVER_BASES:
             if c!=ServerCore:
                 c.threaded_setup(self)
         log("threaded_init() end")
 
+    def wait_for_threaded_init(self):
+        assert self.init_thread
+        log("wait_for_threaded_init() %s.is_alive()=%s", self.init_thread, self.init_thread.is_alive())
+        if self.init_thread.is_alive():
+            log.info("waiting for video encoders initialization")
+            self.init_thread.join(INIT_THREAD_TIMEOUT)
+            if self.init_thread.is_alive():
+                log.warn("Warning: initialization thread is still active")
+
 
     def server_is_ready(self):
         ServerCore.server_is_ready(self)
@@ -136,7 +179,7 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
     def do_cleanup(self):
         self.server_event("exit")
         self.cancel_mp3_stream_check_timer()
-        for c in ServerBase.__bases__:
+        for c in SERVER_BASES:
             if c!=ServerCore:
                 c.cleanup(self)
 
@@ -274,7 +317,8 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
                 self.antialias = c.dictget("antialias")
                 self.cursor_size = c.intget("cursor.size", 0)
             #FIXME: this belongs in DisplayManager!
-            log("dpi=%s, dpi.x=%s, dpi.y=%s, double_click_time=%s, double_click_distance=%s, antialias=%s, cursor_size=%s", self.dpi, self.xdpi, self.ydpi, self.double_click_time, self.double_click_distance, self.antialias, self.cursor_size)
+            screenlog("dpi=%s, dpi.x=%s, dpi.y=%s, antialias=%s, cursor_size=%s", self.dpi, self.xdpi, self.ydpi, self.antialias, self.cursor_size)
+            log("double-click time=%s, distance=%s", self.double_click_time, self.double_click_distance)
             #if we're not sharing, reset all the settings:
             reset = share_count==0
             self.update_all_server_settings(reset)
@@ -286,29 +330,14 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
 
         def drop_client(reason="unknown", *args):
             self.disconnect_client(proto, reason, *args)
-        get_window_id = self._window_to_id.get
-        bandwidth_limit = self.get_client_bandwidth_limit(proto)
-        ClientConnectionClass = self.get_server_source_class()
-        ss = ClientConnectionClass(proto, drop_client,
-                          self.session_name,
-                          self.idle_add, self.timeout_add, self.source_remove, self.setting_changed,
-                          self.idle_timeout,
-                          self._socket_dir, self.unix_socket_paths, not is_request, self.dbus_control,
-                          self.get_transient_for, self.get_focus, self.get_cursor_data,
-                          get_window_id,
-                          self.window_filters,
-                          self.file_transfer,
-                          self.supports_mmap, self.mmap_filename, self.min_mmap_size,
-                          bandwidth_limit,
-                          self.av_sync,
-                          self.core_encodings, self.encodings, self.default_encoding, self.scaling_control,
-                          self.webcam_enabled, self.webcam_device, self.webcam_encodings,
-                          self.sound_properties,
-                          self.sound_source_plugin,
-                          self.supports_speaker, self.supports_microphone,
-                          self.speaker_codecs, self.microphone_codecs,
-                          self.default_quality, self.default_min_quality,
-                          self.default_speed, self.default_min_speed)
+        cc_class = self.get_client_connection_class()
+        ss = cc_class(proto, drop_client,
+                      self.session_name, self,
+                      self.idle_add, self.timeout_add, self.source_remove,
+                      self.setting_changed,
+                      self._socket_dir, self.unix_socket_paths, not is_request,
+                      self.bandwidth_limit, self.bandwidth_detection,
+                      )
         log("process_hello clientconnection=%s", ss)
         try:
             ss.parse_hello(c)
@@ -316,76 +345,56 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
             #close it already
             ss.close()
             raise
-        try:
-            self.notify_new_user(ss)
-        except Exception as e:
-            notifylog("%s(%s)", self.notify_new_user, ss, exc_info=True)
-            notifylog.error("Error: failed to show notification of user login:")
-            notifylog.error(" %s", e)
+        self.notify_new_user(ss)
         self._server_sources[proto] = ss
         #process ui half in ui thread:
         send_ui = ui_client and not is_request
-        self.idle_add(self.parse_hello_ui, ss, c, auth_caps, send_ui, share_count)
+        self.idle_add(self._process_hello_ui, ss, c, auth_caps, send_ui, share_count)
 
-    def notify_new_user(self, ss):
-        #tell other users:
-        notifylog("notify_new_user(%s) sources=%s", ss, self._server_sources)
-        if not self._server_sources:
-            return
-        nid = XPRA_NEW_USER_NOTIFICATION_ID
-        icon = parse_image_path(get_icon_filename("user"))
-        title = "User '%s' connected to the session" % (ss.name or ss.username or ss.uuid)
-        body = "\n".join(ss.get_connect_info())
-        for s in self._server_sources.values():
-            s.notify("", nid, "Xpra", 0, "", title, body, [], {}, 10*1000, icon)
-        
-
-    def get_server_source_class(self):
+    def get_client_connection_class(self):
         from xpra.server.source.client_connection import ClientConnection
         return ClientConnection
 
-    def reset_window_filters(self):
-        self.window_filters = []
 
-    def parse_hello_ui(self, ss, c, auth_caps, send_ui, share_count):
+    def _process_hello_ui(self, ss, c, auth_caps, send_ui, share_count):
         #adds try:except around parse hello ui code:
         try:
             if self._closing:
                 raise Exception("server is shutting down")
-            self.do_parse_hello_ui(ss, c, auth_caps, send_ui, share_count)
+
+            self.parse_hello(ss, c, send_ui)
+            #send_hello will take care of sending the current and max screen resolutions
+            root_w, root_h = self.get_root_window_size()
+            self.send_hello(ss, root_w, root_h, auth_caps)
+            self.add_new_client(ss, c, send_ui, share_count)
+            self.send_initial_data(ss, c, send_ui, share_count)
+            self.client_startup_complete(ss)
+
             if self._closing:
                 raise Exception("server is shutting down")
         except Exception as e:
             #log exception but don't disclose internal details to the client
             p = ss.protocol
-            log("parse_hello_ui%s", (ss, c, auth_caps, send_ui, share_count), exc_info=True)
+            log("_process_hello_ui%s", (ss, c, auth_caps, send_ui, share_count), exc_info=True)
             log.error("Error: processing new connection from %s:", p or ss)
             log.error(" %s", e)
             if p:
                 self.disconnect_client(p, SERVER_ERROR, "error accepting new connection")
 
-    def do_parse_hello_ui(self, ss, c, auth_caps, send_ui, share_count):
-        #process screen size (if needed)
-        if send_ui:
-            root_w, root_h = self.parse_screen_info(ss)
-            self.parse_hello_ui_clipboard(ss, c)
-            key_repeat = self.parse_hello_ui_keyboard(ss, c)
-            self.parse_hello_ui_window_settings(ss, c)
-            if self.notifications_forwarder:
-                client_notification_actions = dict((s.uuid,s.send_notifications_actions) for s in self._server_sources.values())
-                notifylog("client_notification_actions=%s", client_notification_actions)
-                self.notifications_forwarder.support_actions = any(v for v in client_notification_actions.values())
-        else:
-            root_w, root_h = self.get_root_window_size()
-            key_repeat = (0, 0)
+    def parse_hello(self, ss, c, send_ui):
+        for bc in SERVER_BASES:
+            if bc!=ServerCore:
+                bc.parse_hello(self, ss, c, send_ui)
 
-        #send_hello will take care of sending the current and max screen resolutions
-        self.send_hello(ss, root_w, root_h, key_repeat, auth_caps)
+    def add_new_client(self, ss, c, send_ui, share_count):
+        for bc in SERVER_BASES:
+            if bc!=ServerCore:
+                bc.add_new_client(self, ss, c, send_ui, share_count)
 
-        if send_ui:
-            self.send_initial_windows(ss, share_count>0)
-            self.send_initial_cursors(ss, share_count>0)
-        self.client_startup_complete(ss)
+    def send_initial_data(self, ss, c, send_ui, share_count):
+        for bc in SERVER_BASES:
+            if bc!=ServerCore:
+                bc.send_initial_data(self, ss, c, send_ui, share_count)
 
     def client_startup_complete(self, ss):
         ss.startup_complete()
@@ -422,19 +431,17 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
                 "event_request",
                 "notify-startup-complete",
                 "server-events",
-                #newer flags:
-                "av-sync",
                 ))
-        for c in ServerBase.__bases__:
+        for c in SERVER_BASES:
             if c!=ServerCore:
                 merge_dicts(f, c.get_server_features(self, server_source))
         return f
 
     def make_hello(self, source):
         capabilities = ServerCore.make_hello(self, source)
-        for c in ServerBase.__bases__:
+        for c in SERVER_BASES:
             if c!=ServerCore:
-                merge_dicts(capabilities, c.get_caps(self))
+                merge_dicts(capabilities, c.get_caps(self, source))
         capabilities["server_type"] = "base"
         if source.wants_display:
             capabilities.update({
@@ -442,14 +449,14 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
                  })
         if source.wants_features:
             capabilities.update({
-                 "bell"                         : self.bell,
-                 "cursors"                      : self.cursors,
-                 "av-sync.enabled"              : self.av_sync,
                  "client-shutdown"              : self.client_shutdown,
                  "sharing"                      : self.sharing is not False,
                  "sharing-toggle"               : self.sharing is None,
                  "lock"                         : self.lock is not False,
                  "lock-toggle"                  : self.lock is None,
+                 "windows"                      : server_features.windows,
+                 "keyboard"                     : server_features.input_devices,
+                 "pointer"                      : server_features.input_devices,
                  })
             capabilities.update(flatten_dict(self.get_server_features(source)))
         #this is a feature, but we would need the hello request
@@ -457,26 +464,26 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
         capabilities["exit_server"] = True
         return capabilities
 
-    def send_hello(self, server_source, root_w, root_h, key_repeat, server_cipher):
+    def send_hello(self, server_source, root_w, root_h, server_cipher):
         capabilities = self.make_hello(server_source)
         if server_source.wants_encodings:
-            updict(capabilities, "encoding", codec_versions, "version")
-            for k,v in self.get_encoding_info().items():
-                if k=="":
-                    k = "encodings"
-                else:
-                    k = "encodings.%s" % k
-                capabilities[k] = v
+            try:
+                from xpra.codecs.loader import codec_versions
+            except ImportError:
+                log("no codecs", exc_info=True)
+            else:
+                updict(capabilities, "encoding", codec_versions, "version")
+                for k,v in self.get_encoding_info().items():
+                    if k=="":
+                        k = "encodings"
+                    else:
+                        k = "encodings.%s" % k
+                    capabilities[k] = v
         if server_source.wants_display:
             capabilities.update({
                          "actual_desktop_size"  : (root_w, root_h),
                          "root_window_size"     : (root_w, root_h),
-                         "desktop_size"         : self._get_desktop_size_capability(server_source, root_w, root_h),
                          })
-        if key_repeat:
-            capabilities.update({
-                     "key_repeat"           : key_repeat,
-                     "key_repeat_modifiers" : True})
         if self._reverse_aliases and server_source.wants_aliases:
             capabilities["aliases"] = self._reverse_aliases
         if server_cipher:
@@ -492,19 +499,17 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
         ss = self._server_sources.get(proto)
         if not ss:
             return
-        window_ids, categories = [], None
+        categories = None
         #if len(packet>=2):
         #    uuid = packet[1]
-        if len(packet)>=3:
-            window_ids = packet[2]
         if len(packet)>=4:
-            categories = packet[3]
+            categories = tuple(bytestostr(x) for x in packet[3])
         def info_callback(_proto, info):
             assert proto==_proto
             if categories:
                 info = dict((k,v) for k,v in info.items() if k in categories)
             ss.send_info_response(info)
-        self.get_all_info(info_callback, proto, None, window_ids)
+        self.get_all_info(info_callback, proto, None)
 
     def send_hello_info(self, proto, flatten=True):
         start = monotonic_time()
@@ -512,38 +517,34 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
             self.do_send_info(proto, info, flatten)
             end = monotonic_time()
             log.info("processed %s info request from %s in %ims", ["structured", "flat"][flatten], proto._conn, (end-start)*1000)
-        self.get_all_info(cb, proto, None, self._id_to_window.keys())
+        self.get_all_info(cb, proto, None)
 
-    def get_ui_info(self, _proto, _client_uuids=None, wids=None, *_args):
+    def get_ui_info(self, proto, client_uuids=None, *args):
         """ info that must be collected from the UI thread
             (ie: things that query the display)
         """
         info = {"server"    : {"max_desktop_size"   : self.get_max_screen_size()}}
-        if self.keyboard_config:
-            info["keyboard"] = {"state" : {"modifiers"          : self.keyboard_config.get_current_mask()}}
-        #window info:
-        self.add_windows_info(info, wids)
+        for c in SERVER_BASES:
+            try:
+                merge_dicts(info, c.get_ui_info(self, proto, client_uuids, *args))
+            except Exception:
+                log.error("Error gathering UI info on %s", c, exc_info=True)
         return info
 
     def get_thread_info(self, proto):
         return get_thread_info(proto, tuple(self._server_sources.keys()))
 
 
-    def get_info(self, proto=None, client_uuids=None, wids=None, *args):
-        log("ServerBase.get_info%s", (proto, client_uuids, wids, args))
+    def get_info(self, proto=None, client_uuids=None):
+        log("ServerBase.get_info%s", (proto, client_uuids))
         start = monotonic_time()
         info = ServerCore.get_info(self, proto)
-        server_info = info.setdefault("server", {})
-        if self.mem_bytes:
-            server_info["total-memory"] = self.mem_bytes
         if client_uuids:
             sources = [ss for ss in self._server_sources.values() if ss.uuid in client_uuids]
         else:
             sources = tuple(self._server_sources.values())
-        if not wids:
-            wids = self._id_to_window.keys()
-        log("info-request: sources=%s, wids=%s", sources, wids)
-        dgi = self.do_get_info(proto, sources, wids)
+        log("info-request: sources=%s", sources)
+        dgi = self.do_get_info(proto, sources)
         #ugly alert: merge nested dictionaries,
         #ie: do_get_info may return a dictionary for "server" and we already have one,
         # so we update it with the new values
@@ -553,7 +554,6 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
                 info[k] = v
                 continue
             cval.update(v)
-        info.setdefault("cursor", {}).update({"size" : self.cursor_size})
         log("ServerBase.get_info took %.1fms", 1000.0*(monotonic_time()-start))
         return info
 
@@ -568,27 +568,25 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
 
     def get_features_info(self):
         i = {
-             "randr"            : self.randr,
-             "cursors"          : self.cursors,
-             "bell"             : self.bell,
              "sharing"          : self.sharing is not False,
              "idle_timeout"     : self.idle_timeout,
              }
         i.update(self.get_server_features())
         return i
 
-    def do_get_info(self, proto, server_sources=None, window_ids=None):
+    def do_get_info(self, proto, server_sources=None):
         start = monotonic_time()
         info = {}
         def up(prefix, d):
             merge_dicts(info, {prefix : d})
 
-        for c in ServerBase.__bases__:
+        for c in SERVER_BASES:
             try:
                 merge_dicts(info, c.get_info(self, proto))
             except Exception as e:
-                log("do_get_info%s", (proto, server_sources, window_ids), exc_info=True)
-                log.error("Error collecting information from %s: %s", c, e)
+                log("do_get_info%s", (proto, server_sources), exc_info=True)
+                log.error("Error collecting information from %s", c)
+                log.error(" %s", e)
 
         up("features",  self.get_features_info())
         up("network", {
@@ -606,13 +604,11 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
         if n==1:
             ss = server_sources[0]
             up("client", ss.get_info())
-            info.update(ss.get_window_info(window_ids))
         elif n>1:
             cinfo = {}
             for i, ss in enumerate(server_sources):
                 sinfo = ss.get_info()
                 sinfo["ui-driver"] = self.ui_driver==ss.uuid
-                sinfo.update(ss.get_window_info(window_ids))
                 cinfo[i] = sinfo
             up("client", cinfo)
         log("ServerBase.do_get_info took %ims", (monotonic_time()-start)*1000)
@@ -638,6 +634,7 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
                 if v is None:
                     log.warn("removing invalid None property for %s", k)
                     continue
+                k = strtobytes(k)
                 if not k.startswith(b"encoding"):
                     ncp[k] = v
             if ncp:
@@ -653,18 +650,6 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
         for ss in tuple(self._server_sources.values()):
             ss.send_setting_change(setting, value)
 
-    def _process_set_cursors(self, proto, packet):
-        assert self.cursors, "cannot toggle send_cursors: the feature is disabled"
-        ss = self._server_sources.get(proto)
-        if ss:
-            ss.send_cursors = bool(packet[1])
-
-    def _process_set_bell(self, proto, packet):
-        assert self.bell, "cannot toggle send_bell: the feature is disabled"
-        ss = self._server_sources.get(proto)
-        if ss:
-            ss.send_bell = bool(packet[1])
-
     def _process_set_deflate(self, proto, packet):
         level = packet[1]
         log("client has requested compression level=%s", level)
@@ -788,7 +773,7 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
     ######################################################################
     # client connections:
     def init_sockets(self, sockets):
-        for c in ServerBase.__bases__:
+        for c in SERVER_BASES:
             c.init_sockets(self, sockets)
 
     def cleanup_protocol(self, protocol):
@@ -796,48 +781,64 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
         #this ensures that from now on we ignore any incoming packets coming
         #from this connection as these could potentially set some keys pressed, etc
         try:
-            del self._potential_protocols[protocol]
-        except:
+            self._potential_protocols.remove(protocol)
+        except ValueError:
             pass
-        source = self._server_sources.get(protocol)
+        source = self._server_sources.pop(protocol, None)
         if source:
             self.cleanup_source(source)
-            try:
-                del self._server_sources[protocol]
-            except:
-                pass
-        for c in ServerBase.__bases__:
+        for c in SERVER_BASES:
             c.cleanup_protocol(self, protocol)
         return source
 
     def cleanup_source(self, source):
-        had_client = len(self._server_sources)>0
         self.server_event("connection-lost", source.uuid)
+        remaining_sources = tuple(self._server_sources.values())
         if self.ui_driver==source.uuid:
-            self.ui_driver = None
+            if len(remaining_sources)==1:
+                self.set_ui_driver(remaining_sources[0])
+            else:
+                self.set_ui_driver(None)
         source.close()
-        remaining_sources = [x for x in self._server_sources.values() if x!=source]
         netlog("cleanup_source(%s) remaining sources: %s", source, remaining_sources)
         netlog.info("xpra client %i disconnected.", source.counter)
         has_client = len(remaining_sources)>0
-        if had_client and not has_client:
+        if not has_client:
             self.idle_add(self.last_client_exited)
 
     def last_client_exited(self):
         #must run from the UI thread (modifies focus and keys)
+        netlog("last_client_exited() exit_with_client=%s", self.exit_with_client)
+        self.reset_server_timeout(True)
+        for c in SERVER_BASES:
+            if c!=ServerCore:
+                try:
+                    c.last_client_exited(self)
+                except Exception:
+                    log("last_client_exited calling %s", c.last_client_exited, exc_info=True)
         if self.exit_with_client:
-            netlog.info("Last client has disconnected, terminating")
-            self.clean_quit(False)
+            if not self._closing:
+                netlog.info("Last client has disconnected, terminating")
+                self.clean_quit(False)
+
+
+    def set_ui_driver(self, source):
+        if source and self.ui_driver==source.uuid:
+            return
+        log("new ui driver: %s", source)
+        if not source:
+            self.ui_driver = None
         else:
-            self.reset_server_timeout(True)
-            #so it is now safe to clear them:
-            #(this may fail during shutdown - which is ok)
-            try:
-                self.clear_keys_pressed()
-            except:
-                log("last_client_exited", exc_info=True)
-            self._focus(None, 0, [])
-            self.reset_icc_profile()
+            self.ui_driver = source.uuid
+        for c in SERVER_BASES:
+            if c!=ServerCore:
+                c.set_session_driver(self, source)
+
+
+    def reset_focus(self):
+        for c in SERVER_BASES:
+            if c!=ServerCore:
+                c.reset_focus(self)
 
 
     def get_all_protocols(self):
@@ -869,11 +870,9 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
     ######################################################################
     # packets:
     def init_packet_handlers(self):
-        for c in ServerBase.__bases__:
+        for c in SERVER_BASES:
             c.init_packet_handlers(self)
         self._authenticated_packet_handlers.update({
-            "set-cursors":                          self._process_set_cursors,
-            "set-bell":                             self._process_set_bell,
             "sharing-toggle":                       self._process_sharing_toggle,
             "lock-toggle":                          self._process_lock_toggle,
           })
@@ -917,11 +916,14 @@ class ServerBase(ServerCore, ServerBaseControlCommands, NotificationForwarder, W
                 ss = self._server_sources.get(proto)
                 if not self._closing and not proto._closed and (ss is None or not ss.is_closed()):
                     netlog("invalid packet: %s", packet)
-                    netlog.error("unknown or invalid packet type: %s from %s", packet_type, proto)
+                    netlog.error("Error: unknown or invalid packet type '%s'", packet_type)
+                    netlog.error(" received from %s", proto)
                 if not ss:
                     proto.close()
             self.idle_add(invalid_packet)
         except KeyboardInterrupt:
             raise
-        except:
-            netlog.error("Unhandled error while processing a '%s' packet from peer using %s", packet_type, handler, exc_info=True)
+        except Exception:
+            netlog.error("Error processing a '%s' packet", packet_type)
+            netlog.error(" received from %s:", proto)
+            netlog.error(" using %s", handler, exc_info=True)
