@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2017-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2017-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -7,7 +7,7 @@ import sys
 import os.path
 
 from xpra.util import envbool
-from xpra.os_util import OSX, POSIX, shellsub, getuid, getgid, get_util_logger, osexpand
+from xpra.os_util import OSX, POSIX, shellsub, getuid, getgid, get_util_logger, osexpand, umask_context
 from xpra.platform.dotxpra import norm_makepath
 from xpra.scripts.config import InitException
 
@@ -32,6 +32,11 @@ def xpra_runner_shell_script(xpra_file, starting_dir, socket_dir):
         #XPRA_SOCKET_DIR is a special case, it is handled below
         if var=="XPRA_SOCKET_DIR":
             continue
+        if var=="XPRA_ALT_PYTHON_RETRY":
+            #the environment might have changed,
+            #and we may need to retry with a different interpreter
+            #different from the one that created this script
+            continue
         if var.startswith("BASH_FUNC"):
             #some versions of bash will apparently generate functions
             #that cannot be reloaded using this script
@@ -123,12 +128,12 @@ def write_runner_shell_scripts(contents, overwrite=True):
         # Write out a shell-script so that we can start our proxy in a clean
         # environment:
         try:
-            with open(scriptpath, "w") as scriptfile:
-                # Unix is a little silly sometimes:
-                umask = os.umask(0)
-                os.umask(umask)
-                os.fchmod(scriptfile.fileno(), 0o700 & ~umask)
-                scriptfile.write(contents.encode())
+            with umask_context(0o022):
+                h = os.open(scriptpath, os.O_WRONLY|os.O_CREAT|os.O_TRUNC, 0o700)
+                try:
+                    os.write(h, contents.encode())
+                finally:
+                    os.close(h)
         except Exception as e:
             log("writing to %s", scriptpath, exc_info=True)
             log.error("Error: failed to write script file '%s':", scriptpath)
@@ -292,21 +297,27 @@ def get_uinput_device_path(device):
                             return dev_path
     except Exception as e:
         log.error("Error: cannot query uinput device path:")
-        log.error(" %", e)
+        log.error(" %s", e)
     return None
 
 def has_uinput():
     try:
         import uinput
         assert uinput
-    except (ImportError, NameError) as e:
+    except NameError as e:
+        log = get_util_logger()
+        log("has_uinput()", exc_info=True)
+        log.warn("Warning: the system python uinput module looks broken:")
+        log.warn(" %s", e)
+        return False
+    except ImportError as e:
         log = get_util_logger()
         log("has_uinput()", exc_info=True)
         log.info("cannot access python uinput module:")
         log.info(" %s", e)
         return False
     try:
-        uinput.fdopen()
+        uinput.fdopen()         #@UndefinedVariable
     except Exception as e:
         log = get_util_logger()
         log("has_uinput()", exc_info=True)
@@ -347,34 +358,33 @@ def create_uinput_device(uuid, uid, events, name):
 def create_uinput_pointer_device(uuid, uid):
     if not envbool("XPRA_UINPUT_POINTER", True):
         return
-    import uinput
-    events = (
-        uinput.REL_X,
-        uinput.REL_Y,
-        uinput.REL_WHEEL,
-        #REL_HIRES_WHEEL = 0x10
-        #uinput.REL_HWHEEL,
-        uinput.BTN_LEFT,
-        uinput.BTN_RIGHT,
-        uinput.BTN_MIDDLE,
-        uinput.BTN_SIDE,
-        uinput.BTN_EXTRA,
-        uinput.BTN_FORWARD,
-        uinput.BTN_BACK,
+    from uinput import (
+        REL_X, REL_Y, REL_WHEEL,                    #@UnresolvedImport
+        BTN_LEFT, BTN_RIGHT, BTN_MIDDLE, BTN_SIDE,  #@UnresolvedImport
+        BTN_EXTRA, BTN_FORWARD, BTN_BACK,           #@UnresolvedImport
         )
+    events = (
+        REL_X, REL_Y, REL_WHEEL,
+        BTN_LEFT, BTN_RIGHT, BTN_MIDDLE, BTN_SIDE,
+        BTN_EXTRA, BTN_FORWARD, BTN_BACK,
+        )
+    #REL_HIRES_WHEEL = 0x10
+    #uinput.REL_HWHEEL,
     name = "Xpra Virtual Pointer %s" % uuid
     return create_uinput_device(uuid, uid, events, name)
 
 def create_uinput_touchpad_device(uuid, uid):
     if not envbool("XPRA_UINPUT_TOUCHPAD", False):
         return
-    import uinput
+    from uinput import (
+        BTN_TOUCH, ABS_X, ABS_Y, ABS_PRESSURE,      #@UnresolvedImport
+        )
     events = (
-        uinput.BTN_TOUCH,
-        uinput.ABS_X + (0, 2**24-1, 0, 0),
-        uinput.ABS_Y + (0, 2**24-1, 0, 0),
-        uinput.ABS_PRESSURE + (0, 255, 0, 0),
-        #uinput.BTN_TOOL_PEN,
+        BTN_TOUCH,
+        ABS_X + (0, 2**24-1, 0, 0),
+        ABS_Y + (0, 2**24-1, 0, 0),
+        ABS_PRESSURE + (0, 255, 0, 0),
+        #BTN_TOOL_PEN,
         )
     name = "Xpra Virtual Touchpad %s" % uuid
     return create_uinput_device(uuid, uid, events, name)
