@@ -1,22 +1,26 @@
 # This file is part of Xpra.
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import gtk
-import gobject
-
 from xpra.gtk_common.gobject_util import one_arg_signal
-from xpra.x11.gtk_x11.prop import prop_set, prop_get
 from xpra.gtk_common.error import xswallow, xsync
-
+from xpra.x11.gtk_x11.prop import prop_set, prop_get
+from xpra.gtk_common.gobject_compat import import_gdk, import_gobject, is_gtk3
+from xpra.gtk_common.gtk_util import (
+    display_get_default, get_default_root_window, get_xwindow, GDKWindow, x11_foreign_new,
+    STRUCTURE_MASK, EXPOSURE_MASK, PROPERTY_CHANGE_MASK,
+    )
 from xpra.x11.bindings.window_bindings import constants, X11WindowBindings #@UnresolvedImport
 X11Window = X11WindowBindings()
 
 from xpra.x11.gtk_x11.gdk_bindings import (
-               add_event_receiver,                          #@UnresolvedImport
-               remove_event_receiver,                       #@UnresolvedImport
-               )
+    add_event_receiver,                          #@UnresolvedImport
+    remove_event_receiver,                       #@UnresolvedImport
+    )
+
+gdk = import_gdk()
+gobject = import_gobject()
 
 from xpra.log import Logger
 log = Logger("x11", "tray")
@@ -67,10 +71,10 @@ IGNORED_MESSAGE_TYPES = ("_GTK_LOAD_ICONTHEMES", )
 
 
 def get_tray_window(tray_window):
-    return tray_window.get_data(XPRA_TRAY_WINDOW_PROPERTY)
+    return getattr(tray_window, XPRA_TRAY_WINDOW_PROPERTY, None)
 
 def set_tray_window(tray_window, window):
-    tray_window.set_data(XPRA_TRAY_WINDOW_PROPERTY, window.xid)
+    setattr(tray_window, XPRA_TRAY_WINDOW_PROPERTY, get_xwindow(window))
 
 def set_tray_visual(tray_window, gdk_visual):
     prop_set(tray_window, TRAY_VISUAL, "visual", gdk_visual)
@@ -100,19 +104,24 @@ class SystemTray(gobject.GObject):
 
     def cleanup(self):
         log("SystemTray.cleanup()")
-        root = gtk.gdk.get_default_root_window()
-        owner = X11Window.XGetSelectionOwner(SELECTION)
-        if owner==self.tray_window.xid:
-            X11Window.XSetSelectionOwner(0, SELECTION)
-            log("SystemTray.cleanup() reset %s selection owner to %#x", SELECTION, X11Window.XGetSelectionOwner(SELECTION))
-        else:
-            log.warn("SystemTray.cleanup() we were no longer the selection owner")
-        remove_event_receiver(self.tray_window, self)
+        root = get_default_root_window()
         def undock(window):
             log("undocking %s", window)
-            X11Window.Unmap(window.xid)
-            X11Window.Reparent(window.xid, root.xid, 0, 0)
-        for window, tray_window in self.tray_windows.items():
+            wxid = get_xwindow(window)
+            rxid = get_xwindow(root)
+            X11Window.Unmap(wxid)
+            X11Window.Reparent(wxid, rxid, 0, 0)
+        with xswallow:
+            owner = X11Window.XGetSelectionOwner(SELECTION)
+            if owner==get_xwindow(self.tray_window):
+                X11Window.XSetSelectionOwner(0, SELECTION)
+                log("SystemTray.cleanup() reset %s selection owner to %#x", SELECTION, X11Window.XGetSelectionOwner(SELECTION))
+            else:
+                log.warn("Warning: we were no longer the tray selection owner")
+        remove_event_receiver(self.tray_window, self)
+        tray_windows = self.tray_windows
+        self.tray_windows = {}
+        for window, tray_window in tray_windows.items():
             with xswallow:
                 undock(window)
             tray_window.destroy()
@@ -121,42 +130,57 @@ class SystemTray(gobject.GObject):
         log("SystemTray.cleanup() done")
 
     def setup_tray_window(self):
-        display = gtk.gdk.display_get_default()
-        root = gtk.gdk.get_default_root_window()
+        display = display_get_default()
+        root = get_default_root_window()
         screen = root.get_screen()
-        if TRANSPARENCY:
-            colormap, visual = screen.get_rgba_colormap(), screen.get_rgba_visual()
-        if colormap is None or visual is None:
-            log.warn("setup tray: using rgb visual fallback")
-            colormap, visual = screen.get_rgb_colormap(), screen.get_rgb_visual()
-        assert colormap is not None and visual is not None, "failed to obtain visual or colormap"
         owner = X11Window.XGetSelectionOwner(SELECTION)
         log("setup tray: current selection owner=%#x", owner)
         if owner!=XNone:
             raise Exception("%s already owned by %s" % (SELECTION, owner))
-        self.tray_window = gtk.gdk.Window(root, width=1, height=1,
-                                           window_type=gtk.gdk.WINDOW_TOPLEVEL,
-                                           event_mask = 0,
-                                           wclass=gtk.gdk.INPUT_OUTPUT,
-                                           title="Xpra-SystemTray",
-                                           visual=visual,
-                                           colormap=colormap)
-        xtray = self.tray_window.xid
+        kwargs = {}
+        visual = screen.get_system_visual()
+        if TRANSPARENCY:
+            visual = screen.get_rgba_visual()
+            if visual is None:
+                log.warn("setup tray: using rgb visual fallback")
+                visual = screen.get_rgb_visual()
+        #only GTK2 uses colormap:
+        if not is_gtk3():
+            if TRANSPARENCY:
+                colormap = screen.get_rgba_colormap()
+            else:
+                colormap = screen.get_rgb_colormap()
+            kwargs["colormap"] = colormap
+        assert visual is not None, "failed to obtain visual"
+        self.tray_window = GDKWindow(root, width=1, height=1,
+                                  title="Xpra-SystemTray",
+                                  visual=visual,
+                                  **kwargs)
+        xtray = get_xwindow(self.tray_window)
         set_tray_visual(self.tray_window, visual)
         set_tray_orientation(self.tray_window, TRAY_ORIENTATION_HORZ)
         log("setup tray: tray window %#x", xtray)
-        display.request_selection_notification(SELECTION)
-        setsel = X11Window.XSetSelectionOwner(xtray, SELECTION)
-        log("setup tray: set selection owner returned %s, owner=%#x", setsel, X11Window.XGetSelectionOwner(SELECTION))
-        event_mask = StructureNotifyMask
-        log("setup tray: sending client message")
-        X11Window.sendClientMessage(root.xid, root.xid, False, event_mask, "MANAGER",
-                          CurrentTime, SELECTION, xtray)
-        owner = X11Window.XGetSelectionOwner(SELECTION)
-        #FIXME: cleanup if we fail!
-        assert owner==xtray, "we failed to get ownership of the tray selection"
-        add_event_receiver(self.tray_window, self)
-        log("setup tray: done")
+        if is_gtk3():
+            display.request_selection_notification(gdk.Atom.intern(SELECTION, False))
+        else:
+            display.request_selection_notification(SELECTION)
+        try:
+            with xsync:
+                setsel = X11Window.XSetSelectionOwner(xtray, SELECTION)
+                log("setup tray: set selection owner returned %s, owner=%#x", setsel, X11Window.XGetSelectionOwner(SELECTION))
+                event_mask = StructureNotifyMask
+                log("setup tray: sending client message")
+                xid = get_xwindow(root)
+                X11Window.sendClientMessage(xid, xid, False, event_mask, "MANAGER",
+                                  CurrentTime, SELECTION, xtray)
+                owner = X11Window.XGetSelectionOwner(SELECTION)
+                assert owner==xtray, "we failed to get ownership of the tray selection"
+                add_event_receiver(self.tray_window, self)
+                log("setup tray: done")
+        except Exception:
+            log("setup_tray failure", exc_info=True)
+            self.cleanup()
+            raise
 
     def do_xpra_client_message_event(self, event):
         if event.message_type=="_NET_SYSTEM_TRAY_OPCODE" and event.window==self.tray_window and event.format==32:
@@ -167,8 +191,14 @@ class SystemTray(gobject.GObject):
             if opcode==SYSTEM_TRAY_REQUEST_DOCK:
                 xid = event.data[2]
                 log("tray docking request from %#x", xid)
-                with xsync:
-                    self.dock_tray(xid)
+                try:
+                    with xsync:
+                        self.dock_tray(xid)
+                except Exception as e:
+                    log("do_xpra_client_message_event(%s)", event, exc_info=True)
+                    log.warn("Warning: failed to dock tray %#x:", xid)
+                    log.warn(" %s", e)
+                    log.warn(" the application may retry later")
             elif opcode==SYSTEM_TRAY_BEGIN_MESSAGE:
                 timeout = event.data[2]
                 mlen = event.data[3]
@@ -186,33 +216,40 @@ class SystemTray(gobject.GObject):
             log.info("do_xpra_client_message_event(%s)", event)
 
     def dock_tray(self, xid):
-        root = gtk.gdk.get_default_root_window()
-        window = gtk.gdk.window_foreign_new(xid)
+        log("dock_tray(%#x)", xid)
+        root = get_default_root_window()
+        window = x11_foreign_new(xid)
         if window is None:
             log.warn("could not find gdk window for tray window %#x", xid)
             return
+        log("dock_tray: root=%s, window=%s", root, window)
         w, h = window.get_geometry()[2:4]
-        event_mask = gtk.gdk.STRUCTURE_MASK | gtk.gdk.EXPOSURE_MASK | gtk.gdk.PROPERTY_CHANGE_MASK
+        log("dock_tray: geometry=%s", (w, h))
+        if w==0 and h==0:
+            log("dock_tray: invalid tray geometry, ignoring this request")
+            return
+        event_mask = STRUCTURE_MASK | EXPOSURE_MASK | PROPERTY_CHANGE_MASK
         window.set_events(event_mask=event_mask)
         add_event_receiver(window, self)
-        w = max(1, min(64, w))
-        h = max(1, min(64, h))
+        w = max(1, min(128, w))
+        h = max(1, min(128, h))
         title = prop_get(window, "_NET_WM_NAME", "utf8", ignore_errors=True)
         if title is None:
             title = prop_get(window, "WM_NAME", "latin1", ignore_errors=True)
         if title is None:
             title = ""
-        log("dock_tray(%#x) gdk window=%#x, geometry=%s, title=%s, visual.depth=%s", xid, window.xid, window.get_geometry(), title, window.get_visual().depth)
-        event_mask = gtk.gdk.STRUCTURE_MASK | gtk.gdk.EXPOSURE_MASK | gtk.gdk.PROPERTY_CHANGE_MASK
-        tray_window = gtk.gdk.Window(root, width=w, height=h,
-                                           window_type=gtk.gdk.WINDOW_TOPLEVEL,
+        xid = get_xwindow(root)
+        log("dock_tray(%#x) gdk window=%#x, geometry=%s, title=%s", xid, xid, window.get_geometry(), title)
+        kwargs = {}
+        if not is_gtk3():
+            kwargs["colormap"] = window.get_colormap()
+        tray_window = GDKWindow(root, width=w, height=h,
                                            event_mask = event_mask,
-                                           wclass=gtk.gdk.INPUT_OUTPUT,
                                            title=title,
                                            x=-200, y=-200,
                                            override_redirect=True,
                                            visual=window.get_visual(),
-                                           colormap=window.get_colormap())
+                                           **kwargs)
         log("dock_tray(%#x) setting tray properties", xid)
         set_tray_window(tray_window, window)
         tray_window.show()
@@ -220,13 +257,13 @@ class SystemTray(gobject.GObject):
         self.window_trays[tray_window] = window
         log("dock_tray(%#x) resizing and reparenting", xid)
         window.resize(w, h)
-        xwin = window.xid
-        xtray = tray_window.xid
+        xwin = get_xwindow(window)
+        xtray = get_xwindow(tray_window)
         X11Window.Withdraw(xwin)
         X11Window.Reparent(xwin, xtray, 0, 0)
         X11Window.MapRaised(xwin)
         log("dock_tray(%#x) new tray container window %#x", xid, xtray)
-        tray_window.invalidate_rect(gtk.gdk.Rectangle(width=w, height=h), True)
+        tray_window.invalidate_rect(gdk.Rectangle(width=w, height=h), True)
         X11Window.send_xembed_message(xwin, XEMBED_EMBEDDED_NOTIFY, 0, xtray, XEMBED_VERSION)
 
     def move_resize(self, window, x, y, w, h):
