@@ -1,159 +1,27 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-
+from xpra.os_util import bytestostr, strtobytes, hexstr
 from xpra.util import nonl, typedict, envbool, iround
-
-from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
-X11Keyboard = X11KeyboardBindings()
 from xpra.gtk_common.error import xswallow, xsync
-
+from xpra.x11.x11_server_core import X11ServerCore, XTestPointerDevice
+from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
 from xpra.log import Logger
+
 log = Logger("x11", "server")
 mouselog = Logger("x11", "server", "mouse")
 screenlog = Logger("server", "screen")
 
-from xpra.util import envint
-from xpra.os_util import hexstr
-from xpra.x11.x11_server_core import X11ServerCore, XTestPointerDevice
+X11Keyboard = X11KeyboardBindings()
 
-MOUSE_WHEEL_CLICK_MULTIPLIER = envint("XPRA_MOUSE_WHEEL_CLICK_MULTIPLIER", 30)
 SCALED_FONT_ANTIALIAS = envbool("XPRA_SCALED_FONT_ANTIALIAS", False)
 
 
-class UInputDevice(object):
-
-    def __init__(self, device, device_path):
-        self.device = device
-        self.device_path = device_path
-        self.wheel_delta = 0
-        #the first event always goes MIA:
-        #http://who-t.blogspot.co.at/2012/06/xi-21-protocol-design-issues.html
-        #so synthesize a dummy one now:
-        try:
-            with xsync:
-                from xpra.x11.bindings.xi2_bindings import X11XI2Bindings
-                xi2 = X11XI2Bindings()
-                v = xi2.get_xi_version()
-                log("XInput version %s", ".".join(str(x) for x in v))
-                if v<=(2, 2):
-                    self.wheel_motion(4, 1)
-        except:
-            log.warn("cannot query XInput protocol version", exc_info=True)
-
-    def click(self, button, pressed, *_args):
-        import uinput
-        BUTTON_STR = {
-            uinput.BTN_LEFT     : "BTN_LEFT",
-            uinput.BTN_RIGHT    : "BTN_RIGHT",
-            uinput.BTN_MIDDLE   : "BTN_MIDDLE",
-            uinput.BTN_SIDE     : "BTN_SIDE",
-            uinput.BTN_EXTRA    : "BTN_EXTRA",
-            uinput.REL_WHEEL    : "REL_WHEEL",
-            }
-        #this multiplier is based on the values defined in 71-xpra-virtual-pointer.rules as:
-        #MOUSE_WHEEL_CLICK_COUNT=360
-        #MOUSE_WHEEL_CLICK_ANGLE=1
-        mult = MOUSE_WHEEL_CLICK_MULTIPLIER
-        if button==4:
-            ubutton = uinput.REL_WHEEL
-            val = 1*mult
-            if pressed: #only send one event
-                return
-        elif button==5:
-            ubutton = uinput.REL_WHEEL
-            val = -1*mult
-            if pressed: #only send one event
-                return
-        else:
-            ubutton = {
-                1   : uinput.BTN_LEFT,
-                3   : uinput.BTN_RIGHT,
-                2   : uinput.BTN_MIDDLE,
-                8   : uinput.BTN_SIDE,
-                9   : uinput.BTN_EXTRA,
-                }.get(button)
-            val = bool(pressed)
-        if ubutton:
-            mouselog("UInput.click(%i, %s) uinput button=%s (%#x), %#x, value=%s", button, pressed, BUTTON_STR.get(ubutton), ubutton[0], ubutton[1], val)
-            self.device.emit(ubutton, val)
-        else:
-            mouselog("UInput.click(%i, %s) uinput button not found - using XTest", button, pressed)
-            X11Keyboard.xtest_fake_button(button, pressed)
-
-    def wheel_motion(self, button, distance):
-        if button in (4, 5):
-            val = distance*MOUSE_WHEEL_CLICK_MULTIPLIER
-        else:
-            log.warn("Warning: %s", self)
-            log.warn(" cannot handle wheel motion %i", button)
-            log.warn(" this event has been dropped")
-            return
-        delta = self.wheel_delta+val
-        mouselog("UInput.wheel_motion(%i, %.4f) REL_WHEEL: %s+%s=%s", button, distance, self.wheel_delta, val, delta)
-        ival = int(delta)
-        if ival!=0:
-            import uinput
-            self.device.emit(uinput.REL_WHEEL, ival)
-            self.wheel_delta += val-ival
-
-    def close(self):
-        pass
-
-    def has_precise_wheel(self):
-        return True
-
-class UInputPointerDevice(UInputDevice):
-
-    def __repr__(self):
-        return "UInput pointer device %s" % self.device_path
-
-    def move_pointer(self, screen_no, x, y, *_args):
-        mouselog("UInputPointerDevice.move_pointer(%i, %s, %s)", screen_no, x, y)
-        #calculate delta:
-        with xsync:
-            cx, cy = X11Keyboard.query_pointer()
-            mouselog("X11Keyboard.query_pointer=%s, %s", cx, cy)
-            dx = x-cx
-            dy = y-cy
-            mouselog("delta(%s, %s)=%s, %s", cx, cy, dx, dy)
-        import uinput
-        #self.device.emit(uinput.ABS_X, x, syn=(dy==0))
-        #self.device.emit(uinput.ABS_Y, y, syn=True)
-        if dx or dy:
-            if dx!=0:
-                self.device.emit(uinput.REL_X, dx, syn=(dy==0))
-            if dy!=0:
-                self.device.emit(uinput.REL_Y, dy, syn=True)
-
-class UInputTouchpadDevice(UInputDevice):
-
-    def __init__(self, device, device_path, root_w, root_h):
-        UInputDevice.__init__(self, device, device_path)
-        self.root_w = root_w
-        self.root_h = root_h
-
-    def __repr__(self):
-        return "UInput touchpad device %s" % self.device_path
-
-    def move_pointer(self, screen_no, x, y, *_args):
-        mouselog("UInputTouchpadDevice.move_pointer(%i, %s, %s)", screen_no, x, y)
-        import uinput
-        self.device.emit(uinput.BTN_TOUCH, 1, syn=False)
-        self.device.emit(uinput.ABS_X, x*(2**24)//self.root_w, syn=False)
-        self.device.emit(uinput.ABS_Y, y*(2**24)//self.root_h, syn=False)
-        self.device.emit(uinput.ABS_PRESSURE, 255, syn=False)
-        self.device.emit(uinput.BTN_TOUCH, 0, syn=True)
-        with xsync:
-            cx, cy = X11Keyboard.query_pointer()
-            mouselog("X11Keyboard.query_pointer=%s, %s", cx, cy)
-
-
 def _get_antialias_hintstyle(antialias):
     hintstyle = antialias.strget("hintstyle", "").lower()
     if hintstyle in ("hintnone", "hintslight", "hintmedium", "hintfull"):
@@ -163,9 +31,9 @@ def _get_antialias_hintstyle(antialias):
     contrast = antialias.intget("contrast", -1)
     if contrast>1600:
         return "hintfull"
-    elif contrast>1000:
+    if contrast>1000:
         return "hintmedium"
-    elif contrast>0:
+    if contrast>0:
         return "hintslight"
     return "hintnone"
 
@@ -219,6 +87,7 @@ class X11ServerBase(X11ServerCore):
             uinput_device = pointer.get("uinput")
             device_path = pointer.get("device")
             if uinput_device:
+                from xpra.x11.uinput_device import UInputPointerDevice
                 self.input_devices = "uinput"
                 self.pointer_device = UInputPointerDevice(uinput_device, device_path)
                 self.verify_uinput_pointer_device()
@@ -226,6 +95,7 @@ class X11ServerBase(X11ServerCore):
             uinput_device = touchpad.get("uinput")
             device_path = touchpad.get("device")
             if uinput_device:
+                from xpra.x11.uinput_device import UInputTouchpadDevice
                 root_w, root_h = self.get_root_window_size()
                 self.touchpad_device = UInputTouchpadDevice(uinput_device, device_path, root_w, root_h)
         try:
@@ -265,20 +135,26 @@ class X11ServerBase(X11ServerCore):
             screenlog("%i UI clients, resetting ICC profile to default", len(ui_clients))
             self.reset_icc_profile()
             return
-        icc = ui_clients[0].icc
+        icc = typedict(ui_clients[0].icc)
         data = None
         for x in ("data", "icc-data", "icc-profile"):
-            if x in icc:
-                data = icc.get(x)
+            data = icc.strget(x)
+            if data:
                 break
         if not data:
             screenlog("no icc data found in %s", icc)
             self.reset_icc_profile()
             return
-        screenlog("set_icc_profile() icc data for %s: %s (%i bytes)", ui_clients[0], hexstr(data or ""), len(data or ""))
+        screenlog("set_icc_profile() icc data for %s: %s (%i bytes)",
+                  ui_clients[0], hexstr(data or ""), len(data or ""))
         from xpra.x11.gtk_x11.prop import prop_set
         #each CARD32 contains just one 8-bit value - don't ask me why
-        prop_set(self.root_window, "_ICC_PROFILE", ["u32"], [ord(x) for x in data])
+        def o(x):
+            try:
+                return ord(x)
+            except:
+                return x
+        prop_set(self.root_window, "_ICC_PROFILE", ["u32"], [o(x) for x in data])
         prop_set(self.root_window, "_ICC_PROFILE_IN_X_VERSION", "u32", 0*100+4) #0.4 -> 0*100+4*1
 
     def reset_icc_profile(self):
@@ -309,14 +185,14 @@ class X11ServerBase(X11ServerCore):
         #almost like update_all, except we use the default_dpi,
         #since this is called before the first client connects
         self.do_update_server_settings({
-            "resource-manager"  : "",
-            "xsettings-blob"    : (0, [])
+            b"resource-manager"  : b"",
+            b"xsettings-blob"    : (0, [])
             }, reset = True, dpi = self.default_dpi, cursor_size=24)
 
     def update_all_server_settings(self, reset=False):
         self.update_server_settings({
-            "resource-manager"  : "",
-            "xsettings-blob"    : (0, []),
+            b"resource-manager"  : b"",
+            b"xsettings-blob"    : (0, []),
             }, reset=reset)
 
     def update_server_settings(self, settings=None, reset=False):
@@ -343,13 +219,14 @@ class X11ServerBase(X11ServerCore):
                     log.warn(" %s", e)
         old_settings = dict(self._settings)
         log("server_settings: old=%s, updating with=%s", nonl(old_settings), nonl(settings))
-        log("overrides: dpi=%s, double click time=%s, double click distance=%s", dpi, double_click_time, double_click_distance)
+        log("overrides: dpi=%s, double click time=%s, double click distance=%s",
+            dpi, double_click_time, double_click_distance)
         log("overrides: antialias=%s", antialias)
         self._settings.update(settings)
         for k, v in settings.items():
             #cook the "resource-manager" value to add the DPI and/or antialias values:
-            if k=="resource-manager" and (dpi>0 or antialias or cursor_size>0):
-                value = v.decode("utf-8")
+            if k==b"resource-manager" and (dpi>0 or antialias or cursor_size>0):
+                value = bytestostr(v)
                 #parse the resources into a dict:
                 values={}
                 options = value.split("\n")
@@ -370,7 +247,7 @@ class X11ServerBase(X11ServerCore):
                 if antialias:
                     ad = typedict(antialias)
                     subpixel_order = "none"
-                    sss = self._server_sources.values()
+                    sss = tuple(self._server_sources.values())
                     if len(sss)==1:
                         #only honour sub-pixel hinting if a single client is connected
                         #and only when it is not using any scaling (or overriden with SCALED_FONT_ANTIALIAS):
@@ -390,12 +267,13 @@ class X11ServerBase(X11ServerCore):
                 for vk, vv in values.items():
                     value += "%s:\t%s\n" % (vk, vv)
                 #record the actual value used
-                self._settings["resource-manager"] = value
+                self._settings[b"resource-manager"] = value
                 v = value.encode("utf-8")
 
             #cook xsettings to add various settings:
             #(as those may not be present in xsettings on some platforms.. like win32 and osx)
-            if k=="xsettings-blob" and (self.double_click_time>0 or self.double_click_distance!=(-1, -1) or antialias or dpi>0):
+            if k==b"xsettings-blob" and \
+            (self.double_click_time>0 or self.double_click_distance!=(-1, -1) or antialias or dpi>0):
                 from xpra.x11.xsettings_prop import XSettingsTypeInteger, XSettingsTypeString
                 def set_xsettings_value(name, value_type, value):
                     #remove existing one, if any:
@@ -433,8 +311,10 @@ class X11ServerBase(X11ServerCore):
                 def root_set(p):
                     from xpra.x11.gtk_x11.prop import prop_set
                     log("server_settings: setting %s to %s", nonl(p), nonl(v))
-                    prop_set(self.root_window, p, "latin1", v.decode("utf-8"))
-                if k == "xsettings-blob":
+                    prop_set(self.root_window, p, "latin1", strtobytes(v).decode("latin1"))
+                if k == b"xsettings-blob":
                     self.set_xsettings(v)
-                elif k == "resource-manager":
+                elif k == b"resource-manager":
                     root_set("RESOURCE_MANAGER")
+                else:
+                    log.warn("Warning: unexpected setting '%s'", bytestostr(k))
